// $Id: component_registry.js,v 1.29 2003/02/02 11:48:33 russcoon Exp $
// Copyright (c) 2000-2003 Alex Russell
// Licensed under the Academic Free License version 1.1

// Filename:		component_registry.js
// Purpose:		define the component registry for the netWindows environment.
// Classes:		NW_componentRegistry, NW_componentObj
// Global Objects:	__glob_comp_reg__ (singleton)
// Dependencies:	none

if(window.__scripts__){
	__scripts__.provide(__config__.corePath+"component_registry.js");
}

__components__ = new Object();
// FIXME: can we use with() syntax for this?
__components__.regArr = new Array();
__components__.uniqueId = 0;
__components__.register = function(componentObj){
	this.regArr[componentObj.globIdNum] = componentObj;
}

__components__.unregister = function(componentObj){
	//since JS arrays are sparse, don't resize, leave it to GC
	this.regArr[componentObj.globIdNum] = null;
}

__components__.getUniqueIdNum = function(){
	this.uniqueId++;
	return this.uniqueId;
}

__components__.isValidComp = function(compIdNum){
	if(this.regArr[compIdNum] != null){
		return true;
	}else{
		return false;
	}
}

__components__.getComponent = function(compIdNum){
	if(this.isValidComp(compIdNum)){
		return this.regArr[compIdNum]; //return copy of reference to object
	}else{
		return null;
	}
}

__components__.getComponentFromNode = function(domNode){
	if(domNode){ //did we try to select anything at all?
		if(domNode.nodeType == 1){ //only element nodes are of interest
			if(domNode.getAttribute("NW_compID")){ //check for attribute set by all factory functions
				//obtain a reference to the corresponding component object
				compVar = (this.getComponent(domNode.getAttribute("NW_compID")));
				return compVar;
			}else if(domNode == document.body){ //bubbled to top, short-circuit
				return null;
			}else{ //keep bubbling up
				return this.getComponentFromNode(domNode.parentNode);
			}
		}else{ //keep bubbling up
			return this.getComponentFromNode(domNode.parentNode);
		}
	}else{ //nice try there...
		return null;
	}
}

__components__.getComponentOfTypeFromNode = function(domNode, type){
	if((domNode)&&(type)){
		var cmpnt = this.getComponentFromNode(domNode);
		if(cmpnt){
			var par = cmpnt.getParentOfType(type);
			return par; // can be null on error
		}else{
			return null;
		}
	}else{
		return null;
	}
}

__components__.getComponentNodeFromNode = function(domNode){ //thin wrapper
	return this.getComponentFromNode(domNode).domNode;
}

__components__.getComponentsOfType = function(cType){
	var retArr = new Array();
	for(x in this.regArr){
		if(this.regArr[x].componentType==cType){
			retArr[retArr.length]=this.regArr[x];
		}
	}
	if(retArr.length == 0){
		return null;
	}
	return retArr;
}

// make an alias for older code
var __glob_comp_reg__ = __components__;


// the following code defines the class and methods for the componentObj class
function NW_componentObj(domNode){
	// set values for the component that are common to all components.
	this.globIdNum = __components__.getUniqueIdNum();
	this.globId = this.globIdNum;
	this.domNode = domNode;
	__components__.register(this);
	this.componentType = null;
	this.isTopLevelComp = null;
	//links to parent and child components (not DOM nodes)
	this.parentComp = null;
	this.childComp = null;
	this.childComps = new Array();
	this.parseProps = new Array();
	this.setParent = function(pcmp){this.parentComp=pcmp;}
	this.setChild = function(ccmp){this.childComp=ccmp;}
	this.addChild = function(ccmp){this.childComps[this.childComps.length]=ccmp; this.childComps.length++;}
	this.rmChild = function(ccmp){delete this.childComps[ccmp];}
	this.minWidth = null;
	this.minHeight = null;
	this.maxWidth = null;
	this.maxHeight = null;

	this.getTopLevelComponent = function(){
		if(this.isTopLevelComp){
			return this;
		}else{
			var parent_node = null;
			if(this.domNode){
				parent_node = this.domNode.parentNode;
			}else{
				return document.body;
			}
			var temp_c = __components__.getComponentFromNode(parent_node);
			if(temp_c){
				if(!temp_c.isTopLevelComp){
					return temp_c.getTopLevelComponent();
				}else{
					return temp_c;
				}
			}else{
				var genComp = new NW_componentObj(document.body);
				return genComp;
			}
		}
	}
	
	this.getParentOfType = function(cType){
		var parN = null;
		if(this.domNode){
			parN  = this.domNode.parentNode;
		}else{
			return null;
		}
		var temp_c = __components__.getComponentFromNode(parN);
		if(temp_c){
			if(temp_c.componentType != cType){
				return temp_c.getParentOfType(cType);
			}else{
				return temp_c;
			}
		}else{
			return null;
		}
	}

	this.remove = function(){ //remove component's DOM node and application object 
		document.body.removeChild(this.domNode);
		__components__.unregister(this);
	}

	this.registerFocus = function(){
		//subclasses should override this method, providing actions to accomplish when focus in given
		return true;
	}

	this.unRegisterFocus = function(){
		//clean up focus settings when no longer valid
		return true;
	}

	this.doOnPlacement = function(){
		// this method should be called whenever a component is placed in a document
		return true;
	}
}

if(window.__scripts__){
	__scripts__.finalize(__config__.corePath+"component_registry.js");
}
