// $Id: environ_core.js,v 1.173 2003/03/03 06:54:16 russcoon Exp $
// Copyright (c) 2000-2003 Alex Russell
// Licensed under the Academic Free License version 1.1

// Filename:		environ_core.js
// Purpose:		Global Variables and associated Functions
// Classes:		NW_environment
// Global Objects:	__env__
// Dependencies:	component_registry.js, widget_core.js, theme_core.js, netWindows.js

with(__scripts__){
	provide(__config__.corePath+"environ_core.js");
	require(__config__.corePath+"sigslot_core.js");
	require(__config__.corePath+"utility_core.js");
	require(__config__.corePath+"component_registry.js");
	require(__config__.corePath+"widget_core.js");
	require(__config__.corePath+"theme_core.js");
}

function NW_envrionment(){
	// this main purpose of this class is to more-or-less keep the global
	// namespace clean so that we can do things more logicaly

	this.sel_comp = null; //keep the selected component handy
	this.sel_dn = null;
	this.compWithFocus = null;
	this.alphaFocus = null; //set this to true to enable alpha-mediated focus
	this.focusLayer = 0; //set the current topmost layer.
	this.lastX = 0;
	this.lastY = 0;
	// the widget offsets signify the difference from the selected point and the upper leftmost part of the widget
	this.widgetOffsetX = 0;
	this.widgetOffsetY = 0;
	this.currentZ = 0;// the current "highest" layer in the environment
	// browser vars
	this.is_IE = null;
	this.is_mac = null;
	this.is_konq = null;
	this.ctxt = ""; // for table bug in IE > 5.0
	this.docWidth = 0;
	this.docHeight = 0;
	// drag/resize
	this.doDrag = false;
	this.doResize = false;
	this.dragAndSelectDisabled = false;
	this.resizeDir = "";
	
	// content loading
	this.isPageInProcess = false;
	this.pageLoadTarget = 0; //this should be a NW unique ID as given by the component registry
	this.contentQueue = new Array();
	this.currTargetNode = null;
	this.isThemeLoading = false;
	this.timerArray = new Array();
	this.contTimerArray = new Array();
	this.contentTimeout = 15000;//15 seconds
	this.contentFrame = null;
	this.helpCloneEvtHdlrs = true;

	// environment initialization
	this.init = function(){
		if(this.is_IE_50_pc){
			var contFrame = "<div id='contentFrame' name='contentFrame' style='position: absolute; left: 0px; top: 0px; width: 1px; height: 1px; visibility: hidden;'></div>";
			document.body.innerHTML += contFrame;
			this.contentFrame = document.all.contentFrame;
		}else{
			var cframe = document.createElement("iframe");
			this.contentFrame = cframe;
			with(this.contentFrame){
				name = "contentFrame";
				setAttribute("name", "contentFrame");
				id = "contentFrame";
			}
			with(this.contentFrame.style){
				position = "absolute";
				left = "0px";
				top = "0px";
				height = "1px";
				width = "1px";
				visibility = "hidden";
			}
			document.body.appendChild(this.contentFrame);
			this.addNodeEvtHdlr(this.contentFrame, 'load', __util__.cancelEvent, true);
			if(!this.is_konq){
				if(frames.contentFrame){
					frames.contentFrame.location.replace("about:blank");
				}else{
					// for Opera 7
					this.contentFrame.location.replace("about:blank");
				}
			}else{
				setInterval('__env__.toggleVisForKonq();', 1000);
				this.contentFrame.setAttribute("src", "about:blank");
			}
		}

		this.toggleVisForKonq = function(){
			with(this.contentFrame.style){
				if(visibility == "visible"){
					visibility = "hidden";
				}else{
					visibility = "visible";
				}
			}
		}

		this.setDocDims();
		// handle key registration stuff
		this.defaultWidget = new NW_widgetComponent(document.body);
		// for keystrokes registered before init...
		for(var x in this.tkeyarr){
			var y = this.tkeyarr[x];
			this.defaultWidget.keystrokes[this.defaultWidget.keystrokes.length]=y[0];
			__sig__.connect(y[0], y[0].fireKey, y[1], y[2]);
		}
		if(this.is_IE){
			this.helpCloneEvtHdlrs = false;
		}else{
			var nnd = document.createElement("span");
			nnd.innerHTML = "<span onClick='return true;'></span>";
			var tnnd = nnd.cloneNode(true);
			try{
				if(tnnd.firstChild.onclick()){
					stdout("event cloning hack dissabled");
					this.helpCloneEvtHdlrs = false;
				}else{
					stdout("event cloning hack enabled");
				}
			}catch(e){
				stdout("event cloning hack enabled");
			}
		}
		return true;
	}

	// these methods will be emitted from the base events in the browser.
	// Objects wishing to be informed should connect to these functions like so:
	//	__sig__.connect(__env__, __env__.onResize, this, this.resize);
	this.onLoad = function(){return true;}
	this.onResize  = function(){return true;}
	this.onMouseDown = function(){return true;}
	this.onMouseUp = function(){return true;}
	this.onMouseMove = function(){return true;}
	this.onClick = function(){return true;}
	this.onMouseOver = function(){return true;}
	this.onMouseOut = function(){return true;}
	this.keyevtObjs = new Array(); //key events receive special treatment
	this.keyLookup = new Array();
	this.focusChain = new Array();
	// this workaround is needed due to a problem with source inclusion timing
	this.tkeyarr = new Array(); // for key objects that register before the init function fires
	this.defaultWidget = null;

	this.setCurrXY = function(evt){
		this.lastX = this.currentX;
		this.lastY = this.currentY;
		// FIXME: should these include pageX/YOffset and document.body.scrollLeft/Top for Moz and IE respectively?
		if(!evt){ evt = event; }
		this.currentX = evt.clientX;
		this.currentY = evt.clientY;
		this.moveOrResize(evt);
		return false;
	}

	this.resizeNode = function(evt){
		if(this.doResize){
			if(!this.sel_dn){
				var targetNode = __util__.evtTarget(event);
				this.sel_comp = __components__.getComponentFromNode(targetNode);
				this.sel_dn = this.sel_comp.domNode;
			}
			var ixResize = (this.currentX-this.lastX);
			var iyResize = (this.currentY-this.lastY);
			var sds = this.sel_dn.style;
			var xDiff = (parseInt(sds.width) + (this.currentX-this.lastX));
			var yDiff = (parseInt(sds.height) + (this.currentY-this.lastY));
			var ldiff = 0-Math.abs(ixResize);
			var hdiff = 0-Math.abs(iyResize);
			var rszX = true;
			var rszY = true;
			if(this.sel_comp.maxWidth){
				if(xDiff>this.sel_comp.maxWidth){
					if((xDiff-parseInt(sds.width))>0){
						rszX = false;
						ixResize = 0;
					}
				}
			}

			if(this.sel_comp.minWidth){
				if(xDiff<this.sel_comp.minWidth){
					if((xDiff-parseInt(sds.width))<0){
						rszX = false;
						ixResize = 0;
					}
				}
			}

			if(this.sel_comp.maxHeight){
				if(yDiff>this.sel_comp.maxHeight){
					if((yDiff-parseInt(sds.height))>0){
						rszY = false;
						iyResize = 0;
					}
				}
			}

			if(this.sel_comp.minHeight){
				if(yDiff<this.sel_comp.minHeight){
					if((yDiff-parseInt(sds.height))<0){
						rszY = false;
						iyResize = 0;
					}
				}
			}

			if((this.resizeDir.search("N")!=(-1))&&(rszY)){
				sds.top = (parseInt(sds.top)+hdiff)+"px";
				sds.height = (parseInt(sds.height)+(0-iyResize)) + "px";
			}
			if((this.resizeDir.search("E")!=(-1))&&(rszX)){ sds.width = xDiff + "px"; }
			if((this.resizeDir.search("S")!=(-1))&&(rszY)){ sds.height = yDiff + "px"; }
			if((this.resizeDir.search("W")!=(-1))&&(rszX)){
				sds.left = (parseInt(sds.left)+ldiff)+"px";
				sds.width = (parseInt(sds.width)+(0-ixResize)) + "px";
			}
			switch (this.resizeDir){
				case "SE": break;
				case "E": iyResize = 0; break;
				case "S": ixResize = 0; break;
				case "SW": ixResize = (0-ixResize); break;
				case "NW":
					iyResize = (0-iyResize);
					ixResize = (0-ixResize);
					break;
				case "NE": iyResize = (0-iyResize); break;
				case "N":
					ixResize = 0;
					iyResize = (0-iyResize);
					break;
				case "W":
					iyResize = 0;
					ixResize = (0-ixResize);
					break;
				default : break;
			}
			(this.sel_comp.getTopLevelComponent()).resize( ixResize , iyResize );
		}
		return true;
	}


	this.moveNode = function(evt){
		if((this.sel_comp != null)&&(this.doDrag == true)){
			this.sel_dn = this.sel_comp.domNode;
			this.sel_dn.style.left = ((this.currentX*2) - (this.lastX+this.widgetOffsetX)) + "px";
			this.sel_dn.style.top = ((this.currentY*2)-(this.lastY+this.widgetOffsetY)) + "px";
			return true;
		}
	}

	this.moveOrResize = function(evt){
		if(!evt){ evt = event; }
		if(this.doResize){
			this.resizeNode(evt);
			return true;
		}else if((this.sel_comp)&&(this.doDrag)){
			this.moveNode(evt);
			return true;
		}else{ return true; }
	}

	this.selectAndResize = function(evt){
		this.setSelectedComponent(evt);
		this.setResize(evt);
	}

	this.setSelectedComponent = function(evt){
		if(!evt){ evt = event; }
		var targetNode = __util__.evtTarget(evt);
		this.sel_comp = __components__.getComponentFromNode(targetNode);
		if(this.sel_comp){
			if(this.sel_comp.isDraggable){
				this.doDrag = true;
				// make sure we disable dragging and selection durring movement
				this.disableDragAndSelect();
			}else{
				this.doDrag = false;
				this.enableDragAndSelect();

			}
			// allow components to "block" upwards selection bubbling
			if(!this.sel_comp.cancelSelectionBubble){
				this.sel_comp = this.sel_comp.getTopLevelComponent();
				this.sel_dn = this.sel_comp.domNode;
				this.bringToFront();
			}
			var scStyle = this.sel_comp.domNode.style;
			if(this.is_IE){
				var scl = scStyle.pixelLeft; var sct = scStyle.pixelTop;
			}else{
				var scl = scStyle.left; var sct = scStyle.top;
			}
			this.widgetOffsetX = this.currentX - parseInt(scl);
			this.widgetOffsetY = this.currentY - parseInt(sct);
		}else{
			this.enableDragAndSelect();
		}
	}

	this.rmNodeEvtHdlr = function(node, evtName, fp, capture){
		if(node.detachEvent){
			node.detachEvent("on"+evtName, fp);
		}else if((node.removeEventListener)&&(!this.is_konq)){ // Konq 3.1 tries to implement this, and fails
			node.removeEventListener(evtName, fp, capture);
			return true;
		}else{
			eval("node.on"+evtName+"=function(){return true;};");
			return true;
		}
	}

	this.addNodeEvtHdlr = function(node, evtName, fp, capture){
		if(node.attachEvent){
			node.attachEvent("on"+evtName, fp);
		}else if((node.addEventListener)&&(!this.is_konq)){ // Konq 3.1 tries to implement this, but it seems to be broken
			node.addEventListener(evtName, fp, capture);
			return true;
		}else{
			// FIXME: this clobbers the event handler that's already set, is there a way around it?
			eval("node.on"+evtName+"=function(){fp();};");
			return true;
		}
	}


	this.enableDragAndSelect = function(){
		if(this.dragAndSelectDisabled){
			with(document){
				this.rmNodeEvtHdlr(body, 'selectstart', __util__.cancelEvent, true);
				this.rmNodeEvtHdlr(body, 'dragstart', __util__.cancelEvent, true);
			}
			this.dragAndSelectDisabled = false;
		}
	}

	this.disableDragAndSelect = function(){
		if(!this.dragAndSelectDisabled){
			with(document){
				this.addNodeEvtHdlr(body, 'selectstart', __util__.cancelEvent, true);
				this.addNodeEvtHdlr(body, 'dragstart', __util__.cancelEvent, true);
			}
			/*
			// possible mozilla selection fix, does not work here.
			if(!this.is_IE){
				var upEvent = document.createEvent("UIEvents");
				upEvent.initUIEvent("mouseup",0,window,0);
				document.body.dispatchEvent(upEvent);
			}
			*/
			this.dragAndSelectDisabled = true;
		}
	}

	this.unsetSelectedComponent = function(){
		this.sel_comp = null;
		this.sel_dn = null;
		// re-enable dragging and selection if they've been turned off
		this.enableDragAndSelect();	
		this.doDrag = false;
		this.doResize = false;
		this.resizeDir = '';
	}

	this.setResize = function(evt){
		if(!evt){ evt = event; }
		var targetNode = __util__.evtTarget(evt);
		if(targetNode.nodeType==1){
			if(__util__.getAttr(targetNode, "resize")=="true"){
				this.doResize = true;
				this.disableDragAndSelect();
				var sds = this.sel_dn.style;
				if((this.currentY-parseInt(sds.top))<50){
					this.resizeDir = "N";
				}else if(Math.abs(this.currentY-(parseInt(sds.top)+parseInt(sds.height)))<50){
					this.resizeDir = "S";
				}
				if( (this.currentX-parseInt(sds.left))<50){
					this.resizeDir += "W";
				}else if(Math.abs(this.currentX - (parseInt(sds.left)+parseInt(sds.width))) < 50){
					this.resizeDir += "E";
				}
			}else{
				this.doResize = false;
				this.resizeDir = "";
			}
		}else{
			this.doResize = false;
			this.resizeDir = "";
		}
		return false;
	}


	this.setIsIE = function(){
		this.is_konq = navigator.appVersion.indexOf("Konqueror") != -1 ? true : false;
		if(!this.is_konq){
			this.is_konq = navigator.appVersion.indexOf("Safari") == -1 ? false : true; 
		}
		this.is_mac = navigator.appVersion.indexOf("Macintosh") != -1 ? true : false;
		this.is_opera = navigator.userAgent.indexOf("Opera") != -1 ? true : false;
		if((document.all)&&(!this.isOpera)){
			// konq has a document.all collection, and works "bug for bug" in some ways with IE, let it also set is_IE to true
			this.is_IE=true;
			if(navigator.appVersion.indexOf("5.5") != -1){
				this.ctxt = " onclick=\'\'";
			}else if(navigator.appVersion.indexOf("6") != -1){
				this.ctxt = " onclick=\'\'";
			}else{this.ctxt = "";}
		}else{
			this.is_IE=false;
			this.ctxt = "";
		}
		if((!this.is_mac)&&(this.is_IE)&&(navigator.appVersion.indexOf("MSIE 5.0") != -1)){
			this.is_IE_50_pc = true;
		}else{ this.is_IE_50_pc = false; }
		if((!this.is_mac)&&(this.is_IE)&&(navigator.appVersion.indexOf("MSIE 5.5") != -1)){
			this.is_IE_55_pc = true;
		}else{ this.is_IE_55_pc = false; }
		if((!this.is_mac)&&(this.is_IE)&&(navigator.appVersion.indexOf("MSIE 6.0") != -1)){
			this.is_IE_60_pc = true;
		}else{ this.is_IE_60_pc = false; }
	}
	this.setIsIE();//now that we've defined it, call it

	this.setDocDims = function(){
		if(this.is_IE){
			this.docWidth = 0;
			this.docHeight = 0;
			if(document.documentElement){
				this.docWidth = document.documentElement.clientWidth;
				this.docHeight = document.documentElement.clientHeight;
			}
			if(this.docWidth == 0){
				this.docWidth = document.body.clientWidth;
				this.docHeight = document.body.clientHeight;
			}
		}else{
			this.docWidth = window.innerWidth;
			this.docHeight = window.innerHeight;
		}
	}

	this.bringToFront = function(){
		if(this.alphaFocus != null){
			if(this.alphaFocus == true){
				if(this.compWithFocus != null){
					with(this.compWithFocus.domNode.style){
						MozOpacity=("0." + 50);
						filter = ("Alpha(Opacity="+70+")");
					}
				}
			}
			this.compWithFocus = this.sel_comp;
			with(this.compWithFocus.domNode.style){
				MozOpacity=("1.0");
				filter = ("Alpha(Opacity=100)");
			}
		}
		this.currentZ++;
		if(this.sel_dn){
			this.sel_dn.style.zIndex = this.currentZ;
		}
	}

	//functions and variables to controll how pages load.
	this.addToPageQueue = function(URI, dest_div_node){
		var isATheme = false;
		if(arguments.length>2){isATheme = arguments[2];}
		this.contentQueue[(this.contentQueue.length)] = new NW_contentQueueObj(URI, dest_div_node, isATheme);
		this.dequeueNextPage();
	}

	this.dequeueNextPage = NW_dequeueNextPage;
	this.startContentLoad = NW_startContentLoad;
	this.finishContentLoad = NW_finishContentLoad;
	this.isQueueLoaded = function(){
		if(this.contentQueue.length > 0){
			return true;
		}else{
			return false;
		}
	}

	//form functions
	this.submitForm = NW_processForm;
	this.submitRedirectForm = NW_processReTargetedForm;

	//event handler functions and data

	this.setEventHandlers = function(){
		this.addNodeEvtHdlr(window, 'load', this.onloadhdlr, false);
		this.addNodeEvtHdlr(window, 'resize', this.resizehdlr, false);
		this.addNodeEvtHdlr(document, 'mousemove', this.mmovehdlr, false);
		this.addNodeEvtHdlr(document, 'mouseover', this.moverhdlr, false);
		this.addNodeEvtHdlr(document, 'mouseup', this.muphdlr, false);
		this.addNodeEvtHdlr(document, 'mousedown', this.mdownhdlr, false);
		this.addNodeEvtHdlr(document, 'click', this.oclickhdlr, false);
		this.addNodeEvtHdlr(document, 'mouseout', this.mouthdlr, false);
		this.addNodeEvtHdlr(document, 'keypress', this.onkeypress, false);
		/*if(this.is_konq){
			this.addNodeEvtHdlr(window, 'keyup', this.onkeypress, false);
		}*/
		if(this.is_IE){
			this.addNodeEvtHdlr(document, 'keydown', this.onkeydown, false);
		}
	}

	this.keyLookup[13]='enter';
	this.keyLookup[27]='esc';
	this.keyLookup[0]='space';
	this.keyLookup[32]='space';
	this.keyLookup[37]='left';
	this.keyLookup[38]='up';
	this.keyLookup[39]='right';
	this.keyLookup[40]='down';
	this.keyLookup[45]='ins';
	this.keyLookup[46]='del';
	
	this.setKeystroke = function(key, sh, ctl, alt, obj, fp){
		if(this.defaultWidget == null){
			var keyEvtObj = new NW_keyEvt(key, sh, ctl, alt);
			this.tkeyarr[this.tkeyarr.length]=[keyEvtObj, obj, fp];
		}else{
			this.defaultWidget.setKeystroke(key, sh, ctl, alt, obj, fp);
		}
	}

	this.unsetKeystroke = function(key, sh, ctl, alt, obj, fp){
		this.defaultWidget.unsetKeystroke(key, sh, ctl, alt, obj, fp);
	}

	this.setFocusChain = function(evt){
		var chain = new Array();
		var elem = __util__.evtTarget(evt);
		while(elem != document.body){
			while((elem.nodeType != 1)&&(elem != document.body)){
				elem = elem.parentNode;
			}
			if(elem == document.body){ break; }
			var ret = __components__.getComponentFromNode(elem);
			if(ret != null){ // null means we hit body searching from a node
				chain[chain.length]=ret;
				elem = ret.domNode.parentNode;
			}else{
				elem = document.body;
			}
		}
		chain[chain.length]=this.defaultWidget;
		this.focusChain = chain.reverse();
		__sig__.emit(this.focusSet, elem, this.defaultWidget, this.focusChain);
	}

	this.focusSet = function(focusedNode, focusedWidget, chain){
		return true;
	}
	__sig__.addBareSignal(this, this.focusSet);

	//default action functions
	this.mMoveDefault = function(event){ __env__.setCurrXY(event); return true; }
	this.mUpDefault = function(event){ __env__.unsetSelectedComponent(); return true; }
	this.mDownDefault = function(event){ __env__.selectAndResize(event); return true; }
	this.resizeDefault = function(event){ __env__.setDocDims(event); return true; }
	this.onLoadConstructorsFinished = function(){
		return true;
	}
	__sig__.addBareSig(this, this.onLoadConstructorsFinished);

	this.oLoadDefault = function(event){ 
		this.init();
		if((window.NWconfig)&&(NWconfig['searchIds'])){
			if(NWconfig['searchIds'].length>0){
				__config__.searchIds = __config__.searchIds.concat(NWconfig['searchIds']);
			}
		}
		var widl = __config__.searchIds.length;
		if(widl > 0){
			for(var x = 0; x < widl; x++){
				var currNd = document.getElementById(__config__.searchIds[x]);
				NW_checkNodeIsConstructor(currNd, currNd);
			}
		}
		if(__config__.findWidgets){
			NW_helpCloneNodes(document.body, document.body);
		}
		__sig__.emit(__env__.onLoadConstructorsFinished, event);
		return true;
	}

	//callbacks to emitters for mozilla, to be fired from nodes
	this.onloadhdlr  = function(event){__sig__.emit(__env__.onLoad, event);}
	this.mmovehdlr = function(event){__sig__.emit(__env__.onMouseMove, event);}
	this.muphdlr = function(event){__sig__.emit(__env__.onMouseUp, event);}
	this.mdownhdlr = function(event){__sig__.emit(__env__.onMouseDown, event);}
	this.oclickhdlr = function(event){__sig__.emit(__env__.onClick, event);}
	this.moverhdlr = function(event){__sig__.emit(__env__.onMouseOver, event);}
	this.mouthdlr = function(event){__sig__.emit(__env__.onMouseOut, event);}
	this.resizehdlr = function(event){__sig__.emit(__env__.onResize, event);}

	if(this.is_IE){ // IE specific handler code
		this.onkeypress = function(){
			var kc = Number(event.keyCode);
			var k_char = String.fromCharCode(kc).toLowerCase();
			if((65 <= kc)&&(kc <= 90)){
				for(var x = __env__.focusChain.length-1; x >= 0; x--){
					if(__env__.focusChain[x].checkKeys(k_char, event.shiftKey, event.ctrlKey, event.altKey)){
						__env__.focusChain[x].fireKeys(k_char, event.shiftKey, event.ctrlKey, event.altKey);
						break;
					}
				}
			}
			return true;
		}
		
		this.onkeydown = function(){//handles irregular keys
			var kc = Number(event.keyCode);
			if(__env__.keyLookup[kc]){
				var k_char = __env__.keyLookup[kc].toLowerCase();
				for(var x = __env__.focusChain.length-1; x >= 0; x--){
					if(__env__.focusChain[x].checkKeys(k_char, event.shiftKey, event.ctrlKey, event.altKey)){
						__env__.focusChain[x].fireKeys(k_char, event.shiftKey, event.ctrlKey, event.altKey);
						break;
					}
				}
			}else if((65 <= kc)&&(kc <= 90)&&(!event.shiftKey)){
				__env__.onkeypress(event);
			}
		}
		
	}else{ // Mozilla specific handler code
		this.onkeypress = function(e){
			var kc = Number(e.keyCode);
			if((65 <= kc)&&(kc <= 90)){
				var k_char = String.fromCharCode(kc).toLowerCase();
			}else if (kc==0){
				var k_char = String.fromCharCode(e.charCode).toLowerCase();
				if(k_char == " "){
					k_char = "space";
				}
			}else if(__env__.keyLookup[kc]){
				var k_char = __env__.keyLookup[kc].toLowerCase();
			}
			// iterate over the focusChain array and dispatch
			for(var x = __env__.focusChain.length-1; x >= 0; x--){
				if(__env__.focusChain[x].checkKeys(k_char, e.shiftKey, e.ctrlKey, e.altKey)){
					__env__.focusChain[x].fireKeys(k_char, e.shiftKey, e.ctrlKey, e.altKey);
					break;
				}
			}
			return true;
		}

		if(__config__.autoSetHandlers){
			this.setEventHandlers();
		}
	}
}


//create the global environment object
__env__ = new NW_envrionment();
__NW_env__ = __env__;

if(__env__.is_IE){
	if(__config__.autoSetHandlers){
		__env__.setEventHandlers();
	}
}

with(__sig__){
	connect(__env__, __env__.onLoad, __env__, __env__.oLoadDefault);
	connect(__env__, __env__.onMouseMove, __env__, __env__.mMoveDefault);
	connect(__env__, __env__.onMouseUp, __env__, __env__.mUpDefault);
	connect(__env__, __env__.onMouseDown, __env__, __env__.mDownDefault);
	connect(__env__, __env__.onResize, __env__, __env__.resizeDefault);
	connect(__env__, __env__.onClick, __env__, __env__.setFocusChain);
}

function NW_setFocus(event){
	var tgt = __util__.evtTarget(event);
	var comp = __components__.getComponentFromNode(tgt);
	return true;
}


////////////////////////////////////////
// Page/Content load functions

////////////////////////////////////////
// this class will define the data
// structure for a queued page load
function NW_contentQueueObj(fd, dest_n, isTh){
	//add passed elements to the object...
	this.frameDest = fd;
	this.destDiv = dest_n;
	this.isTheme = isTh;
}

function NW_dequeueNextPage(){
	if((this.isPageInProcess)&&(this.isQueueLoaded())){
		this.timerArray[this.timerArray.length] = setTimeout("__env__.dequeueNextPage()", 500);
	}else{
		//clear out the timers we may have set...
		for(i=0; i < this.timerArray.length; i++){
			clearTimeout(this.timerArray[i]);
		}
		this.timerArray = new Array();
		//go ahead and copy the contents of the window over if we can...
		if(this.contentQueue.length>0){
			// stdout(this.contentQueue[0].frameDest);
			if(this.is_IE_50_pc){
				this.startContentLoad(this.contentQueue[0].frameDest, this.contentQueue[0].destDiv, this.contentQueue[0].isTheme);
				var tq = this.contentQueue;
				this.contentQueue = new Array();
				if(tq.length>1){
					for(x=1; x<tq.length; x++){
						this.contentQueue[this.contentQueue.length] = tq[x];
					}
				}
				this.finishContentLoad();
			}else{
				this.startContentLoad(this.contentQueue[0].frameDest, this.contentQueue[0].destDiv, this.contentQueue[0].isTheme);
				this.contentQueue = this.contentQueue.slice(1);
			}
		}else{
			this.contentQueue = new Array();
		}
	}
}

function NW_startContentLoad(frameDest, destDiv, isTheme){
	this.isPageInProcess = true;
	this.currTargetNode = destDiv;
	this.isThemeLoading = isTheme;
	//load the hidden content frame...
	if(this.is_IE_50_pc){
		this.contentFrame.innerHTML = NW_getFileTxt(frameDest);
		return true;
	}else if(this.is_konq){
		this.contentFrame.setAttribute("src", (NW_urlToAbs(frameDest)[2]));
	}else{
		frames.contentFrame.location.replace(NW_urlToAbs(frameDest)[2]);
		//frames.contentFrame.location.replace(frameDest);
		this.contTimerArray[this.contTimerArray.length] = setTimeout("__env__.isPageInProcess=false; __env__.dequeueNextPage()", this.contentTimeout);
	}
}

function NW_finishContentLoad(){
	for(i=(this.contTimerArray.length-1); i >= 0; i--){
		clearTimeout(this.contTimerArray[i]);
	}
	this.contTimerArray = new Array();
	if(this.isThemeLoading){
		__theme__.parseThemeFromFile();
	}else{
		if(this.is_IE_50_pc){
			var contBody = this.contentFrame;
		}else{
			var contBody = frames.contentFrame.document.body;
		}
		var cloned_node = null;
		var temp_cloned_node = null;
		var cl_node_count = contBody.childNodes.length;
		var widgetsArray = new Array();
		
		if(this.currTargetNode){
			while(this.currTargetNode.firstChild){
				this.currTargetNode.removeChild(this.currTargetNode.firstChild);
			}
		}
		//now we go ahead and grab the contents of the page in question.
		if(this.currTargetNode != null){
			this.currTargetNode.innerHTML += contBody.innerHTML;
		}
		//ensure that things are themed
		if(this.currTargetNode != null){
			__theme__.changeTheme(this.currTargetNode);
			NW_helpCloneNodes(this.currTargetNode, this.currTargetNode);
		}
	}

	//if there's something still to be loaded, load it!
	this.isPageInProcess = false;
	if(this.isQueueLoaded()){
		this.dequeueNextPage();
	}else{
		if(this.is_IE_50_pc){
			this.contentFrame.innerHTML = "";
		}else if(this.is_konq){
			this.contentFrame.setAttribute("src", "about:blank");
		}else{
			if(this.is_IE){ // for some reason, this gives Moz strange behaviour
				frames.contentFrame.location.replace("about:blank");
			}
		}
	}
	return true;
}

// this recursive function will set attributes for a node tree
// it's a helper function and doesn't need env objects so we are <i>not</i> going
// to register it as a member function.
function NW_helpCloneNodes(set_n, src_n){

	var snType = src_n.nodeType;
	if(snType == 1){
		if(__env__.helpCloneEvtHdlrs){
			with(src_n){
				var c_mOver = getAttribute("onmouseover");
				var c_mOut = getAttribute("onmouseout");
				var c_mDown = getAttribute("onmousedown");
				var c_mUp = getAttribute("onmouseup");
				var c_mClick = getAttribute("onclick");
				if((c_mOver != null)&&(c_mOver != "")){setAttribute("onMouseOver", c_mOver);}
				if((c_mOut != null)&&(c_mOut != "")){setAttribute("onMouseOut", c_mOut);}
				if((c_mDown != null)&&(c_mDown != "")){setAttribute("onMouseDown", c_mDown);}
				if((c_mUp != null)&&(c_mUp != "")){setAttribute("onMouseUp", c_mUp);}
				if((c_mClick != null)&&(c_mClick != "")){setAttribute("onClick", c_mClick);}
			}
		}

		// recursive JavaScript...heaven help us all =)
		if(src_n.hasChildNodes()){
			for(var tcnt=0; tcnt < set_n.childNodes.length; tcnt++){
				NW_checkNodeIsConstructor(set_n.childNodes.item(tcnt), src_n.childNodes.item(tcnt));
			}
		}
	}
}

function NW_checkNodeIsConstructor(set_n, src_n){
	var snt = src_n.nodeType;
	var params = null;
	var nodeType = null;
	var locObj = null;
	var localNode = null;
	if( (snt <= 12)&&(snt != 3)&&(snt != 8) ){
		if(__util__.hasAttr(src_n,"nwType")){
			var nwAppId = null;
			if(__util__.hasAttr(src_n,"nwAppId")){
				nwAppId = __util__.getAttr(src_n,"nwAppId");
			}
			nodeType = __util__.getAttr(src_n,"nwType");
			// if the widget type isn't valid, error out
			if(!__widgets__.isWidgetComp(nodeType)){ return null; }
			locObj = __widgets__.runInlineConstructor(nodeType, src_n);
			// if we didn't get a valid widget back, error out
			localNode = locObj.domNode;

			// FIXME: do we need some sort of other lookup for auto-id stuff?

			// DEFAULTS
			__util__.prependClass(localNode, nodeType);
			
			// OVER-RIDES
			if(__util__.hasAttr(src_n,"nwProperties")){
				var params = __util__.parsePropStr(__util__.getAttr(src_n,"nwProperties"));
				if(params["nwAppId"]){
					nwAppId = __util__.stripMozIndex(params["nwAppId"]);
				}
				// try to get the placement argument if there is one
				if(params["placement"]){
					var placement = __util__.stripMozIndex(new String(params["placement"]).toLowerCase());
				}else{
					var placement = "inline";
				}
				if(params["auxClass"]){
					__util__.addClass(localNode, params["auxClass"]);
				}
				// make sure the current theme is correctly applied
				__theme__.applyTheme(localNode);
				if(placement == "appendtargetnode"){
					if(__env__.currTargetNode != null){
						__env__.currTargetNode.appendChild(localNode);
					}
				}else if((placement != "inline")&&(__env__.currTargetNode!=null)){
					//add logic to change location here!!
					var tlc = __components__.getComponentFromNode(__env__.currTargetNode).getTopLevelComponent();
					if(tlc.addOtherObjs){
						tlc.addOtherObjs("after", "header", localNode);
					}
				}else{
					set_n.parentNode.replaceChild(localNode, set_n);
				}
			}else{
				set_n.parentNode.replaceChild(localNode, set_n);
			}
			// callback to component to inform it that it's been placed
			if(locObj.doOnPlacement){
				locObj.doOnPlacement();
			}
			if(nwAppId!=null){
				// FIXME: is there a more efficent way to do this?
				nwAppId = new String(nwAppId);
				nwAppId = nwAppId.replace(";","");
				nwAppId = nwAppId.replace(" ","");
				nwAppId = nwAppId.replace('"',"");
				nwAppId = nwAppId.replace("'","");
				eval("window."+nwAppId+"=locObj;");
			}
		}else{
			NW_helpCloneNodes(set_n, src_n);
		}
	}
	delete params;
	delete locObj;
	delete localNode;
	delete nodeType;
	delete placement;
	delete snt;
}

function NW_processForm(form_node, action_page){
	//grab the form element for parsing...
	var final_action = action_page; //where are we submitting to?
	var tpost_str = "?"; //start building post string...
	//make sure the node we're grabbing is actually a form
	while(form_node.nodeName.toLowerCase() != "form"){
		form_node = form_node.parentNode;
	}
	tpost_str += NW_addFormValues(form_node);
	final_action += tpost_str;
	new_final_action = final_action.slice(0, (final_action.length - 1) );
	//support re-targeted forms
	try{
		//if there's a DOM nodes passed in as the target, use it
		if(arguments.length>2){
			var final_target = arguments[2];
		//else, use the comopnent the form is in
		}else{
			var final_target = __components__.getComponentFromNode(form_node).domNode;
		}
		this.addToPageQueue(new_final_action, final_target, false);
	}catch(final_target){
		document.location.replace(new_final_action);
	}
}

function NW_processReTargetedForm(from_node, action_page, target_ref){
	NW_processForm(from_node, action_page, terget_ref);
}

function NW_addFormValues(sNode){
	//the argument is a DOM Node corresponding to a form element.
	var tvar = "";
	var ctyp = sNode.nodeName.toLowerCase();
	if ((ctyp=="input") || (ctyp=="select") || (ctyp=="textarea")){
		tvar = sNode.getAttribute("name") + "=" + sNode.value  + "&";
	}
	if(sNode.hasChildNodes()){
		for(var temp_count=(sNode.childNodes.length-1); temp_count >= 0; temp_count--){
			tvar += NW_addFormValues(sNode.childNodes.item(temp_count));
		}
	}
	return tvar;
}

__scripts__.finalize(__config__.corePath+"environ_core.js");
