<!-- 
/* --> 
<chapter>
	<title>netWindows.js</title>
	<sect1>
		<title>__scripts__</title>
		<!-- this is a singleton object, so the class ref is also the object ref -->
		<classsynopsis>
			<ooclass>
				<classname>__scripts__</classname>
				<modifier>singleton</modifier>
			</ooclass>
			<classsynopsisinfo>
				<para>
				The __scripts__ class provides a netWindows 
				environment with facilities for dependency 
				satisfaction and notification.
				</para>
			</classsynopsisinfo>

<!--*/

// $Id: netWindows.js,v 1.102 2003/02/20 08:04:48 russcoon Exp $
// Copyright (c) 2000-2003 Alex Russell
// Licensed under the Academic Free License version 1.1

// Purpose:		path config data, load core scripts
// Classes:		NW_config_core, NW_scriptRegistry
// Global Objects:	__scripts__, __config__
// Dependencies:	none



// NOTE: the following two lines illustrate what's needed to add a new docbook sect
/* --> 
<!--*/

// if someone has included us in degraded JS mode, kill execution
if(!(new Boolean(false))){
	// FIXME: should we warn here with an alert instead?
	throw new Error("Do not use language='JavaScript1.2'");
}

// some fixes for Array deficiencies in some browsers *ahem*
if (typeof Array.prototype.push == 'undefined'){
	Array.prototype.push = function() {
		var cl = this.length;
		for (var i = 0; i < arguments.length; i++) {
			this[this.length] = arguments[i];
		}
		return this.length;
	};
}

if (typeof Array.prototype.pop == 'undefined'){
	Array.prototype.pop = function(){
		var ret = null;
		if (this.length > 0){
			this.length = this.length-1;
			ret = this[this.length];
		}
		return ret;
	};
}


// define the script registry object
// it doesn't need a constructor function, so we create it as a true singleton
__scripts__ = new Object();
__scripts__.isPathLoaded = new Array();
/* --> 
<fieldsynopsis>
	<type>boolean</type>
	<varname>initLoad</varname>
	<initializer>true</initializer>
</fieldsynopsis>
<!--*/
__scripts__.initLoad = true;
/* --> 
<fieldsynopsis>
	<type>string</type>
	<varname>loadStart</varname>
	<initializer>null</initializer>
</fieldsynopsis>
<!--*/
__scripts__.loadStart = null;
/* --> 
<methodsynopsis>
	<!-- return type -->
	<type>boolean</type>
	<!-- name of the method -->
	<methodname>require</methodname>
	<!-- any parameters -->
	<methodparam>string <parameter>path</parameter></methodparam>
</methodsynopsis>
<!--*/
__scripts__.require = function(path){
	var absPath = (NW_urlToAbs(path))[2];
	if(!this.isPathLoaded[absPath]){
		this.isPathLoaded[absPath] = [new Date(), null, []];
		if(this.initLoad){
			/* // FIXME: need to test standards-based script inclusion, such as the following
			var ns = document.createElement("script");
			ns.setAttribute("src",absPath);
			ns.src = absPath;
			ns.setAttribute("type","text/javascript");
			document.getElementsByTagName("head")[0].appendChild(ns);
			*/
			document.write('<script language="JavaScript" src="'+absPath+'" type="text/javascript"><\/script>');
		}else{ 
			NW_includeScript(path);
		}
		return true;
	}
	return false;
}

/* --> 
<methodsynopsis>
	<type>boolean</type>
	<methodname>addDeps</methodname>
	<methodparam>string <parameter>path</parameter></methodparam>
</methodsynopsis>
<para>
	<methodname>__scripts__.addDeps()</methodname> is identical to <methodname>__scripts__.require()</methodname>.
</para>
<!--*/
__scripts__.addDeps = __scripts__.require;

/* --> 
<methodsynopsis>
	<type>null</type>
	<methodname>provide</methodname>
	<methodparam>string <parameter>path</parameter></methodparam>
</methodsynopsis>
<!--*/
__scripts__.provide = function(path){
	var absPath = (NW_urlToAbs(path))[2];
	if(!this.isPathLoaded[absPath]){
		this.isPathLoaded[absPath] = [ (new Date()), null ];
	}
}

/* --> 
<methodsynopsis>
	<type>null</type>
	<methodname>addParents</methodname>
	<methodparam>string <parameter>path</parameter></methodparam>
</methodsynopsis>
<para>
	<methodname>__scripts__.addParents()</methodname> is identical to <methodname>__scripts__.provide()</methodname>.
</para>
<!--*/
__scripts__.addParents = __scripts__.provide;

/* --> 
<methodsynopsis>
	<type>null</type>
	<methodname>doOnFinalize</methodname>
	<methodparam>string <parameter>path</parameter></methodparam>
	<methodparam>function <parameter>fuction_pointer</parameter></methodparam>
</methodsynopsis>
<para>
	allows developers to specify a function to call when the specified path is <methodname>provide</methodname>ed.
</para>
<!--*/
__scripts__.doOnFinalize = function(path, fp){
	var absPath = (NW_urlToAbs(path))[2];
	if(this.isPathLoaded[absPath]!=null){
		this.isPathLoaded[absPath][2][this.isPathLoaded[absPath][2].length]=fp;
	}
}

/* --> 
<methodsynopsis>
	<type>null</type>
	<methodname>finalize</methodname>
	<methodparam>string <parameter>path</parameter></methodparam>
</methodsynopsis>
<para>
	should be called from each included file to signal that it has finished 
	loading and that all objects, classes, functions, and data it provides 
	are available to the runtime.
</para>
<!--*/
__scripts__.finalize = function(path){
	var absPath = (NW_urlToAbs(path))[2];
	if(this.isPathLoaded[absPath]!=null){
		this.isPathLoaded[absPath][1]= new Date();
		for(var x in this.isPathLoaded[absPath][2]){
			this.isPathLoaded[absPath][2][x]();
		}
	}
}

/* --> 
<methodsynopsis>
	<type>null</type>
	<methodname>dumpTimes</methodname>
	<void />
</methodsynopsis>
<para>
	a debugging method that allows developers to 
	view at the debugging console (if available) 
	how much time was required to load each script 
</para>
<!--*/
__scripts__.dumpTimes = function(){
	if(window.stdout){
		for(x in this.isPathLoaded){
			var iplx = this.isPathLoaded[x];
			if(iplx[1]!=null){
				stdout(x);
				stdout((iplx[1]-iplx[0])+"ms total");
				if(this.loadStart != null){
					stdout("started: " + (iplx[0]-this.loadStart));
					stdout("finished: " + (iplx[1]-this.loadStart));
				}
			}
		}
	}
}

/* --> 
<methodsynopsis>
	<type>null</type>
	<methodname>endInitLoads</methodname>
	<void />
</methodsynopsis>
<para>
	invoke XMLHTTP script loading for all 
	subsequent calls to <methodname>__scripts__.require</methodname>
</para>
<!--*/
__scripts__.endInitLoads = function(){
	this.initLoad=false;
}

/* --> 
<methodsynopsis>
	<type>boolean</type>
	<methodname>isLoaded</methodname>
	<methodparam>string <parameter>path</parameter></methodparam>
</methodsynopsis>
<para>determine if a script is already available</para>
<!--*/
__scripts__.isLoaded = function(path){
	var absPath = (NW_urlToAbs(path))[2];
	if(this.isPathLoaded[absPath]!=null){ 
		return false;
	}else{ 
		return true;
	}
}

var __script_registry__ = __scripts__ ;
/* --> 
		</classsynopsis>
	</sect1>
<!--*/

function NW_includeScript(scriptURL){
	__env__.code_to_add = NW_getFileTxt(scriptURL);// place in global in deference to IE
	if(__env__.is_IE){
		var exec_node = document.createElement("<span onClick='eval(__env__.code_to_add);'>");
		document.body.appendChild(exec_node);
		exec_node.fireEvent("onClick");
		//document.body.removeChild(exec_node);
	}else{
		eval(__env__.code_to_add, window);
	}
	__env__.code_to_add = null;
	return true;
}

// FIXME: this should really be encapsulated in an object of some sort. I don't like the namespace polution.
NW_getFileTxtCache = [];
// The basis for the next function is a contribution from Jeff Yates (PBWiz@mail.pbwizard.com)
// Many thanks to him for sending me the code from which this came
// 11/22/01: suggested modifications from Ted Shroyer (edward.shroyer@FedEx.com)
function NW_getFileTxt(url){
	// FIXME: need to do object detection instead of just removing konq support out of hand
	if(!__env__.is_konq){
		var absURLArr= NW_urlToAbs(url); //get's array with info for full path
		var curl = absURLArr[2];
		var doCache = __config__.cacheTxtURLData;
		if(arguments.length>1){
			// the (optional) 2nd argument explicitly determines 
			// whether or not to check for a cached copy and 
			// if we should cache the results
			doCache = arguments[1];
		}
		if((doCache)&&(NW_getFileTxtCache[curl])){
			return NW_getFileTxtCache[curl];
		}else{
			if(__env__.is_IE){
				var p = new ActiveXObject("Microsoft.XMLHTTP");
			}else{
				var p = new XMLHttpRequest();
			}
			p.open("GET", curl, false);
			p.send(null);
			var tdat = p.responseText;
			if(doCache){
				NW_getFileTxtCache[curl] = tdat;
			}
			return tdat;
		}
	}else{
		return "";
	}
}


//this function is copied in it's entriety from a script sent to me by Jeff Yates
//(PBWiz@mail.pbwizard.com)
function NW_urlToAbs(url){
	//get the page's url
	var baseUrl = document.location.toString();
	var bases = document.getElementsByTagName("base");
	if( bases.length > 0 ){
		baseUrl = bases[bases.length-1].href;
	}
	baseUrl = baseUrl.replace(/\\/g, "/").replace(/\s/g, "%20"); 
	url = url.replace(/\\/g, "/").replace(/\s/g, "%20");
	//detect if the provided url is already an absolute url
	if( /^[a-zA-Z]*:\/\//.test( url ) ){ // detected "name://"
		return new Array(url, url, url);
	}
	url = url.replace(/^(.\/)*/, "");
	baseUrl = baseUrl.replace( /(\?.*)$/, "");//query
	baseUrl = baseUrl.replace( /(#.*)*$/, "");//hash
	baseUrl = baseUrl.replace( /[^\/]*$/, "");//file name

	//detect if the provided url is relative from the host root
	if( /^\//.test(url) ){
		baseUrl = baseUrl.replace( /[^:]*$/, "");
		return new Array(baseUrl, url, (baseUrl + url));
	}

	//the following code deals with the parent directory references
	while( /^..\//.test( url ) ){
		url = url.replace( /^..\//, "");
		baseUrl = baseUrl.replace( /([^\/]+[\/][^\/]*)$/, "");
	}
	//return an array here instead of the full path, as Jeff did
	return new Array(baseUrl, url, (baseUrl + url));
}

// configuration class constructor
function NW_config_core(){
	this.NWroot =  "";
	this.corePath = null;
	this.searchIds = [];

	this.toInc = new Array();
	this.addIncURL = function(){
		for(var x = 0; x < arguments.length; x++){
			this.toInc[this.toInc.length] = arguments[x];
		}
	}

	this.addSearchId = function(){
		for(var x = 0; x < arguments.length; x++){
			this.searchIds[this.searchIds.length] = arguments[x];
		}
	}

	// non path-related config options
	this.themeName =	null;
	this.applyThemedStyles = true;
	this.includeTooltips = 	true;
	this.isProduction = 	false;
	this.dhtmlEnabled = 	true;
	this.autoLoad = 	true;
	this.findWidgets =	true;
	this.autoSetHandlers =  true;
	this.obeyRequestEnv =	false;
	this.cacheTxtURLData =	true;
	// name of function to call (with no args) when netWindows.js finishes loading
	this.loadEvalName =	null;

	this.checkAssocArr = function(arr){
		// if(arr.length == 0){ return false; }
		for(x in arr){ 
			switch (x){
				// we could set these with an eval, but we'll do this for speed
				case "widgetPath":
					this.widgetPath = arr[x];
					break;
				case "rootPath":
					this.NWroot = arr[x];
					break;
				case "NWroot":
					this.NWroot = arr[x];
					break;
				case "corePath":
					this.corePath = arr[x];
					break;
				case "imgPath":
					this.imgPath = arr[x];
					break;
				case "appPath":
					this.appPath = arr[x];
					break;
				case "includeTooltips":
					this.includeTooltips = arr[x];
					break;
				case "isProduction":
					this.isProduction = arr[x];
					break;
				case "dhtmlEnabled":
					this.dhtmlEnabled = arr[x];
					break;
				case "autoLoad":
					this.autoLoad = arr[x];
					break;
				case "findWidgets":
					this.findWidgets = arr[x];
					break;
				case "autoSetHandlers":
					this.autoSetHandlers = arr[x];
					break;
				case "searchIds":
					this.searchIds = this.searchIds.concat(arr[x]);
					break;
				case "obeyRequestEnv":
					this.obeyRequestEnv = arr[x];
					break;
				case "themeName":
					this.themeName = arr[x];
					break;
				case "loadEvalName":
					this.loadEvalName = arr[x];
					break;
				case "applyThemedStyles":
					this.applyThemedStyles = arr[x];
					break;
				case "cacheTxtURLData":
					this.cacheTxtURLData = arr[x];
					break;
			}
		}
		return true;
	}

	this.parseGetStr = function(){
		var baseUrl = document.location.toString();
		var params = baseUrl.split("?", 2);
		if(params.length > 1){
			var paramStr = params[1];
			var pairs = paramStr.split("&");
			var opts = new Array();
			for(var x in pairs){
				var sp = pairs[x].split("=");
				// FIXME: is this eval dangerous?
				opts[sp[0]]=eval(sp[1]);
			}
			return opts;
		}else{
			return [];
		}
	}

	// if the associative array "NWconfig" exists,
	// check it's values and set our options accordingly
	if(window.NWconfig){
		this.checkAssocArr(NWconfig);
		window.NWconfig["searchIds"] = [];
	}

	// if allowed, parse options from the GET string of the URI
	if(this.obeyRequestEnv){
		this.checkAssocArr(this.parseGetStr());
	}

	// pull the base path for the NW codebase out of the path this script was included with

	if(this.NWroot==""){
		var scripts = document.getElementsByTagName('script');
		if(this.corePath!=null){
			if(this.corePath.substr(this.corePath.length-1)=="/"){
				this.corePath = this.corePath.substr(0, this.corePath.length-1);
			}
			this.corePath = this.corePath.replace("\/", "\\\/");
			var trestr = new RegExp(this.corePath+"\/netWindows\.js$");
		}else{
			var trestr = new RegExp("winScripts\/netWindows\.js$");
		}

		// the basis for this provided by Tim Morgan(tim@timmorgan.info) and used by permission.
		for(var i=0; i<scripts.length; i++){
			// FIXME: need to add code to check for alternates to "winScripts"
			if((scripts[i].src)&&(scripts[i].src.match(trestr))){ 
				this.NWroot = scripts[i].src.replace(trestr, '');
			}
		}
		if(this.NWroot == ""){
			this.NWroot = "./";
		}
		delete scripts;
		delete trestr;
	}


	// path specific options
	this.corePath = 	this.NWroot + "winScripts/";
	this.imgPath =		this.NWroot + "images/";
	this.appPath =		this.NWroot + "appScripts/";
	this.widgetPath =	this.corePath + "widgets/";

	this.includeScripts = function(){
		// finish environment loading
		if(this.dhtmlEnabled){

			var cp = this.corePath;
			this.addIncURL(cp+"sigslot_core.js");
			this.addIncURL(cp+"environ_core.js");
			if(this.includeTooltips){
				this.addIncURL(cp+"tooltips.js");
			}
			this.addIncURL(cp+"theme_core.js");
			this.addIncURL(cp+"component_registry.js");
			this.addIncURL(cp+"widget_core.js");
			this.addIncURL(cp+"data_core.js");


			// if we're in a production environment, we don't want a real console
			if(this.isProduction){
				this.addIncURL(cp+"fake_console.js");
			}else{
				this.addIncURL(cp+"console.js");
			}

			__scripts__.loadStart = new Date();
			__scripts__.provide(this.corePath+"netWindows.js");
			for(var x=0; x<this.toInc.length; x++){
				__scripts__.require(this.toInc[x]);
			}
		}
	}

}
__config__ = new NW_config_core();
__NW_config__ = __config__;

if(__config__.autoLoad){
	__config__.includeScripts();
}
if(__config__.loadEvalName){
	if(eval("window."+__config__.loadEvalName)){
		eval("window."+__config__.loadEvalName+"();");
	}
}

/*-->
</chapter>
*/
