// $Id: theme_core.js,v 1.40 2003/01/12 09:38:14 russcoon Exp $
// Copyright (c) 2000-2003 Alex Russell
// Licensed under the Academic Free License version 1.1

// Filename:		environ_core.js
// Purpose:		define functions to handle the loading of themes.
// Classes:		NW_themeObject
// Global Objects:	__current_theme__
// Dependencies:	theme_core.js

with(__scripts__){
	provide(__config__.corePath+"theme_core.js");
	require(__config__.corePath+"environ_core.js");
}

__theme__ = new Object();
__theme__.isThemeSet = false;
__theme__.nodeAttr = "theme";
__theme__.defAttr = "themedef";
__theme__.themeAttributes = new Array();
__theme__.themeNames = new Array();

__theme__.addThemeName = function(name, url){
	if(this.themeNames[name]==null){
		this.themeNames[name] = url;
		return true;
	}
	stdout("theme name already used");
}

__theme__.getThemeByName = function(name){
	if(this.themeNames[name]!=null){
		this.getThemeFromFile(this.themeNames[name]);
		return true;
	}
	stdout("invalid theme name given");
}
__theme__.setThemeByName = __theme__.getThemeByName;

__theme__.getThemeFromFile = function(url){ 
	__env__.addToPageQueue(url, null, true);
}
__theme__.setThemeFromFile = __theme__.getThemeFromFile;
__theme__.getThemeAttr = function(attrName){ return this.themeAttributes[attrName]; }
__theme__.getAttr = __theme__.getThemeAttr; // alias function name
__theme__.setAttr = function(attrName, attrData){ this.themeAttributes[attrName] = attrData; }
__theme__.setThemeAttr = __theme__.setAttr;

__theme__.parseThemeFromFile = function(){
	stdout("parsing theme");
	this.themeAttributes.length = 0; //clear the current theme!!
	if(__env__.is_IE_50_pc){
		var themeDOM = __env__.contentFrame.document.getElementById(this.defAttr);
	}else{
		var themeDOM = frames.contentFrame.document.getElementById(this.defAttr);
	}
	if(themeDOM == null){ 
		this.isThemeSet = false;
		stdout("NO THEME LOADED! Aborting!");
		return false;
	}
	//theme files are only 1 level deep.
	for(var key = (themeDOM.childNodes.length-1); key >= 0; key--){
		if(themeDOM.childNodes.item(key).id != null){ //ignore nodes without an ID
			this.themeAttributes[themeDOM.childNodes.item(key).id] = themeDOM.childNodes.item(key).innerHTML;
		}
	}
	this.isThemeSet = true;
	this.changeTheme(document.body);
	if(__env__.isQueueLoaded()){
		// __env__.isPageInProcess = false;
		__env__.dequeueNextPage();
	}
	return true;
}

__theme__.outputTheme = function(){ // debugging output
	stdout(this.isThemeSet);
	for(var key in this.themeAttributes){
		stdout(key + " " + this.themeAttributes[key]);
	}
}

__theme__.modifyTheme = function(styleList, nodeToSet){
	var styleArr = NW_parseExpandoProperties(styleList);
	var tpropsArr = NW_parseExpandoProperties(nodeToSet.getAttribute(__theme__.nodeAttr));
	var props = "";
	for (var tsn in styleArr){
		tpropsArr[tsn]=styleArr[tsn];
	}
	for (var styleName in tpropsArr){
		props += styleName + ":" + tpropsArr[styleName][0] + ";";
	}
	nodeToSet.setAttribute(__theme__.nodeAttr, props);
	delete props;
	delete styleArr;
	delete tpropsArr;
	return true;
}

__theme__.setStyleByAttr = function(nd, attr, prop){
	var sStr = prop +":"+this.getThemeAttr(attr)+";";
	var oStr = prop+":"+attr+";";
	if(__config__.applyThemedStyles){
		__util__.setNodeStyle(sStr, nd);
	}
	if(__util__.hasAttr(nd, this.nodeAttr)){
		this.modifyTheme(oStr, nd);
	}else{
		nd.setAttribute(this.nodeAttr, oStr); 
	}
	return true;
}

// updates the theme on the fly...
__theme__.changeTheme = function(domNode){
	if((__env__.is_IE)&&(!__env__.is_konq)){
		// we are expecting theme attribute info to be in the format similar to:
		// "bgColor: main_border_color; borderLeft: main_border_color;"
		// note that this is a 1:1 mapping
		for(var keys in domNode.childNodes){
			if((domNode.childNodes[keys])&&(domNode.childNodes[keys].nodeType == 1)){
				if(domNode.childNodes[keys].getAttribute(this.nodeAttr)){
					var propsArray = NW_parseExpandoProperties(domNode.childNodes[keys].getAttribute(this.nodeAttr));
					for(var key in propsArray){
						var attrName = new String(propsArray[key]);
						if((attrName.indexOf('img')!=(-1))||(attrName.indexOf('image')!=(-1))){
							domNode.childNodes[keys].src = __config__.NWroot + this.getThemeAttr(attrName);
						}else{
							eval("domNode.childNodes[keys].style."+key+"=this.getThemeAttr(attrName);");
						}
					}
				}
				if(domNode.childNodes[keys].hasChildNodes()){
					this.changeTheme(domNode.childNodes[keys]);
				}
			}
		}
	}else{ 
		for(var keys = (domNode.childNodes.length-1); keys >= 0 ; keys--){
			if((domNode.childNodes[keys].hasChildNodes())&&(domNode.childNodes[keys].nodeType == 1)){
				this.changeTheme(domNode.childNodes[keys]);
			}
			if((domNode.childNodes[keys])&&(domNode.childNodes[keys].nodeType == 1)){
				if(domNode.childNodes[keys].hasAttribute(this.nodeAttr)){
					var propsArray = NW_parseExpandoProperties(domNode.childNodes[keys].getAttribute(this.nodeAttr));
					for(var key in propsArray){
						key = __util__.stripMozIndex(key);
						var attrName = __util__.stripMozIndex(propsArray[key]);
						if((key.indexOf('img')!=(-1))||(key.indexOf('image')!=(-1))){
							eval("domNode.childNodes[keys].src =   __config__.NWroot + this.getThemeAttr(attrName);");
						}else{
							eval("domNode.childNodes[keys].style." + key + " = this.getThemeAttr(attrName);");
						}
					}
				}
			}
		}
	}
	return true;
}


__theme__.applyTheme = __theme__.changeTheme; // function alias

//create a default theme object that is world accessible.
var __current_theme__ = __theme__ ;

with(__theme__){
	// in order to keep things working smoothly for env pages that don't want to
	// load external content (it _is_ a preformance hit) we sure that sane 
	// defaults are set for the most common theme attributes. Change these to 
	// change the "default" colors of widgets in your environment
	setAttr("main_bgcolor", "#F6F6F6");
	setAttr("main_border_color", "#CCCCCC");
	setAttr("main_font_family", "Arial");
	setAttr("main_font_size", "11px");
	setAttr("main_font_color", "black");

	setAttr("head_bar_bgcolor", "#d5d5d5");
	setAttr("head_bar_font_family", "Arial");
	setAttr("head_bar_font_color", "black");
	setAttr("head_bar_font_size", "11px");

	setAttr("content_bgcolor", "white");
	setAttr("content_border_color", "#d0d0d0");
	setAttr("content_font_family", "Arial");
	setAttr("content_font_size", "11px");
	setAttr("content_font_color", "black");

	setAttr("menu_bgcolor", "#F6F6F6");
	setAttr("menu_hl_bgcolor", "#e6e6e6");
	setAttr("menu_border_color", "#F6F6F6");
	setAttr("menu_border_style", "dotted");
	setAttr("menu_font_family", "Arial");
	setAttr("menu_font_color", "black");
	setAttr("menu_hl_font_color", "black");
	setAttr("menu_font_size", "11px");

	var nr = __config__.NWroot;
	setAttr("window_pin_up_image", nr+"themes/newDefault/pin.gif");
	setAttr("window_pin_down_image", nr+"themes/newDefault/pin.gif");
	setAttr("window_close_image", nr+"themes/newDefault/close.gif");
	setAttr("window_min_image", nr+"themes/newDefault/min.gif");
	setAttr("window_max_image", nr+"themes/newDefault/max.gif");
	setAttr("window_drag_image", nr+"themes/newDefault/drag.gif");

	// setup a default list of themes which are available by name
	addThemeName("default", nr+"themes/newDefault/newDefault.html");
	addThemeName("system", nr+"themes/newDefault/newDefault.html");
	addThemeName("blue", nr+"themes/newDefault/blues.html");
	addThemeName("green", nr+"themes/newDefault/greens.html");
	addThemeName("orange", nr+"themes/newDefault/oranges.html");
	addThemeName("yellow", nr+"themes/newDefault/yellows.html");
	addThemeName("red", nr+"themes/newDefault/reds.html");
}

// auto-load a named theme if there is a valid theme name given
if((__config__.themeName!=null)&&(__theme__.themeNames[__config__.themeName]!=null)){
	__sig__.connect(__env__, __env__.onLoad, null, function(){__theme__.setThemeByName(__config__.themeName);});
}

__scripts__.finalize(__config__.corePath+"theme_core.js");
