// $Id: utility_core.js,v 1.117 2003/03/03 14:44:33 russcoon Exp $
// Copyright (c) 2000-2003 Alex Russell
// Licensed under the Academic Free License version 1.1

// Filename:		utility_core.js
// Purpose:		some helper fuctions that really don't belong in environ_core
// Classes:		NW_utility, NW_keyEvt, NW_element_maker_obj, NW_genericParser
// Global Objects:	__util__
// Dependencies:	none, this file should be stand alone, avoid dependencies here

/*
	perhaps the general timing functions from effects_core should be moved here?
*/

if(window.__scripts__){
	__scripts__.provide(__config__.corePath+"utility_core.js");
}

function NW_utility(){
	this.sorts = [];
	this.converters = [];

	/*
	* sorts with converters
	*/

	// case-sensitive string sort has no conversion, and can use native sort.
	this.converters["string:cs"] = null;
	this.sorts["string:cs"] = null;

	// case-insensitive string
        this.converters["string:cis"] = function(a) {return a.toUpperCase();}
        this.sorts["string:cis"] = null;

	// date in any format that Date.parse understands
        this.converters["date:native"] = function(a) {var i = Date.parse(a); return isNaN(i) ? 0 : i;}

        this.sorts["date:native"] = function(a,b) {return a - b;}

	// base-10 integer
	// FIXME: maybe use - Infinity instead of 0 for unparseable strings?
	this.converters["integer:10"] = function(a) {var i = parseInt(a,10); return isNaN(i) ? 0 : i;}
	this.sorts["integer:10"] = function(a,b) {return a - b;}

	/*
	* sorts without converters
	*/
	this.sorts["string"] = function(a, b){
		if(a > b){
			return 1;
		}else if(a < b){
			return -1;
		}else{
			return 0;
		}
	}
	
	this.sorts["string:caseinsensitive"] = function(a, b){
		// FIXME: should is just call this.sorts["string"]() here?
		var ta = a.toUpperCase();
		var tb = b.toUpperCase();
		if(ta > tb){
			return 1;
		}else if(ta < tb){
			return -1;
		}else{
			return 0;
		}
	}
	
	this.sorts["int"] = function(a, b){
		a = __util__.toInt(a);
		b = __util__.toInt(b);
		if(a > b){
			return 1;
		}else if(a < b){
			return -1;
		}else{
			return 0;
		}
	}

	this.sorts["float"] = function(a, b){
		a = __util__.toFloat(a);
		b = __util__.toFloat(b);
		if(a > b){
			return 1;
		}else if(a < b){
			return -1;
		}else{
			return 0;
		}
	}
	
	this.sorts["number"] = this.sorts["float"];

	this.sorts["date:MM/DD/YYYY"] = function(a, b){
		// we assume that the passed in value is just the date string.
		var as = a.split("/");
		var bs = b.split("/");
		if((as.length != 3)||(bs.length != 3)){
			if((as.length == 3)&&(bs.length != 3)){
				return 1;
			}else if((as.length != 3)&&(bs.length == 3)){
				return -1;
			}else{
				return 0; // neither are valid dates
			}
		}
		// MDA's stopgap (hopefuly) performance increasing algo:
		aiso = [as[2], as[0], as[1]].join("");
		biso = [bs[2], bs[0], bs[1]].join("");
		if (aiso > biso){ return 1; }
		if (aiso < biso){ return -1; }
		return 0;
		/*
		with(__util__){
			da1[0] = toInt(da1[0]);
			da1[1] = toInt(da1[1]);
			da1[2] = toInt(da1[2]);
			da2[0] = toInt(da2[0]);
			da2[1] = toInt(da2[1]);
			da2[2] = toInt(da2[2]);
		}
		if(da1[2]>da2[2]){
			return 1;
		}else if(da1[2]==da2[2]){
			if(da1[0]>da2[0]){
				return 1;
			}else if(da1[0]==da2[0]){
				if(da1[1]>da2[1]){
					return 1;
				}else if(da1[1]==da2[1]){
					return 0;
				}else{
					return -1;
				}
			}else{
				return -1;
			}
		}else{
			return -1;
		}
		*/
	}

	this.isKonq = navigator.appVersion.indexOf("Konqueror") == -1 ? false : true; 
	if(!this.isKonq){
		this.isKonq = navigator.appVersion.indexOf("Safari") == -1 ? false : true; 
	}
	this.isMac = navigator.appVersion.indexOf("Macintosh") == -1 ? false : true; 
	this.isOpera = navigator.userAgent.indexOf("Opera") == -1 ? false : true; 
	this.isIE = false;
	if((document.all)&&(!this.isOpera)){ this.isIE=true; }

	this.setNodeStyle = function(styleList, nodeToSet){
		var sArr = NW_parseExpandoProperties(styleList);
		for (key in sArr) {
			var valVar = new String(sArr[key]);
			// valVar = NW_stripMozIndexBug(valVar);
			with(nodeToSet.style){
				if((this.isIE)&&(!this.isKonq)){
					eval("try{nodeToSet.style."+key+"='"+valVar+"';}catch(e){stdout('illegal style key: '+key);}");
				}else{
					if(key!="float"){
						nodeToSet.style[key] = valVar;
					}else{
						nodeToSet.style.cssFloat = valVar;
					}
				}
			}
		}
		return true;
	}

	this.parsePropStr = NW_parseExpandoProperties;
	this.stripWS = NW_stripWhiteSpaceNodes;
	this.stripMozIndex = NW_stripMozIndexBug;
	// this.genParse = NW_genericParser;
	this.genParse = function(nd){return new Object();};
	this.elementMaker = NW_element_maker_obj;
	this.evtTarget = function(event){
		if(event.srcElement){
			return event.srcElement;
		}else{
			return event.target;
		}
	}

	this.isDescendant = function(node, p_ancestor){
		if(node.nodeType==1){
			if(node==p_ancestor){
				return true;
			}else if(node!=document.body){
				return this.isDescendant(node.parentNode, p_ancestor);
			}else{
				return false;
			}
		}
	}

	this.setOpacity = function(node, pct){
		if(document.all){
			node.style.filter = "Alpha(Opacity="+pct+")";	
		}else{
			node.style.MozOpacity = (pct/100);
		}
	}

	// this code donated by Pieter Theron (www.azure-online.com)
	this.cancelEvent = function(evt){
		if(!evt){ return false; }
		if(evt.preventDefault){
			evt.stopPropagation();
			evt.preventDefault();
		}else{
			if(window.event){
				window.event.cancelBubble = true;
			}
		}
		return false;
	}

	this.noNaN = function(val){
		var i = parseInt(val,10);
		return isNaN(i) ? 0 : i;
	}

	this.toInt = this.noNaN;

	this.toFloat = function(val){
		return (isNaN(parseFloat(val))) ? 0.0 : parseFloat(val);
	}

	this.getComputedStyle = function(nodeRef, attr){
		if(nodeRef.currentStyle) { // IE
			if(attr == "width"){
				return nodeRef.offsetWidth;
			}else if(attr == "height"){
				return nodeRef.offsetHeight;
			}
			// otherwise try to get the cascaded _specified_ value
			return nodeRef.currentStyle[this.toCamelCase(attr)];
		}else if(document.defaultView.getComputedStyle){ // spec
			return document.defaultView.getComputedStyle(nodeRef, null).getPropertyValue(attr);
		}else { // borked
			return "0";
		}
	}

	this.getCSVal = function(nodeRef, attr){
		return this.toInt(this.getComputedStyle(nodeRef, attr));
	}

	this.getCSVals = function(nodeRef){
		if(arguments.length > 1){
			var rl = 0;
			// FIXME: reverse order of this iteration?
			for(var x=1; x < arguments.length; x++){
				rl += this.getCSVal(nodeRef, arguments[x]);
			}
			return rl;
		}
		return 0;		
	}

	// stolen from Dave Schontzler (stilleye.com) by way of Tim Scarfe (www.developer-x.com)
	this.toCamelCase = function(str){
		var s = str.split('-');
		var o = [s[0]];

		for(var i=1;j=s[i];i++) { 
			o[o.length] = j.charAt(0).toUpperCase( ); 
			o[o.length] = j.substring(1);
		}
		return o.join('');	
	}

	// inspiration for this code taken from Pieter Theron (www.azure-online.com)
	this.getPBM = function(nodeRef){
		var retArr = new Array();
		if (nodeRef){
			retArr["y"] = this.getCSVals(nodeRef, 'padding-top', 'padding-bottom', 'margin-top', 'margin-bottom', 'border-top-width', 'border-bottom-width');
			retArr["x"] = this.getCSVals(nodeRef, 'padding-left', 'padding-right', 'margin-left', 'margin-right', 'border-left-width', 'border-right-width');
		} else {
			retArr["x"] = retArr["y"] = 0;
		}
		return retArr;
	}

	this.hasAttr = function(node, attr){
		// FIXME: need to add support for attr-specific accessors here
		var ta = new String(attr);
		if(!node){ return null; }
		if(node.getAttribute){
			if((node.getAttributeNode)&&(node.getAttributeNode(ta))&&(node.getAttributeNode(ta).value!="")){
				return true;
			}else if(node.getAttribute(ta)){
				return true;
			}else if(node.getAttribute(ta.toLowerCase())){
				return true;
			}else if(node.getAttribute(ta.toUpperCase())){
				return true;
			}
		}
		return false;
	}

	this.getAttr = function(node, attr){
		// FIXME: need to add support for attr-specific accessors here
		var ta = new String(attr);
		if(!node){ return null; }
		// this should work on Opera 7, but it's a little on the crashy side
		if((node.getAttributeNode)&&(node.getAttributeNode(ta))){
			return (node.getAttributeNode(ta)).value;
		}else if(node.getAttribute(ta)){
			return node.getAttribute(ta);
		}else if(node.getAttribute(ta.toLowerCase())){
			return node.getAttribute(ta.toLowerCase());
		}else if(node.getAttribute(ta.toUpperCase())){
			return node.getAttribute(ta.toUpperCase());
		}
		return null;
	}

	this.getClass = function(node){
		if(node.className){
			return node.className;
		}else if(this.hasAttr(node, "class")){
			return this.getAttr(node, "class");
		}
		return null;
	}

	this.prependClass = function(node, classStr){
		if(this.hasAttr(node,"class")||node.className){
			classStr += " " + (this.getAttr(node, "class")||node.className);
		}
		return this.setClass(node, classStr);
	}

	this.addClass = function(node, classStr){
		if(this.hasAttr(node,"class")||node.className){
			classStr = (this.getAttr(node, "class")||node.className) + " " + classStr;
		}
		return this.setClass(node, classStr);
	}

	this.setClass = function(node, classStr){
		var cs = new String(classStr);
		if(!node){ return false; }
		try{
			node.setAttribute("class", classStr);
			node.className = cs;
		}catch(e){
			stdout(e);
		}
		return true;
	}

	// FIXME: need to add support here for a method of mapping some element hierarchy to a set of component types
	this.parseNWComponents = function(node){
		// get the properties of the current node to use as the base of our return.
		var args = new Array();
		var alen = arguments.length;
		// is there a way to do this with slice()?
		args[args.length]=node;
		for(var x = 1; x < alen; x++){
			args[args.length] = arguments[x];
		}
		var tprops = this.parseNWProps.apply(this, args);
		// now that we have the props array, append child props arrays, recurse to do so.
		if(node.hasChildNodes()){
			for(var x = 0; x < node.childNodes.length; x++){
				var y = node.childNodes[x];
				if(this.hasAttr(y,"nwType")){
					var ctype = this.getAttr(y,"nwType");
					if(!tprops["nwComponents"]){
						tprops["nwComponents"]=new Array();
					}
					if(!tprops["nwComponents"][ctype]){
						tprops["nwComponents"][ctype]=new Array();
					}
					args[0]=y;
					tprops["nwComponents"][ctype][tprops["nwComponents"][ctype].length]=this.parseNWComponents.apply(this, args);
				}
			}
		}
		return tprops;
	}

	this.parseNWProps = function(node){
		// var start = new Date();
		var attrs = new Array();
		var props = new Array();
		var mvald = new Array();
		if(!node){
			return new Array();
		}

		props = this.parsePropStr(this.getAttr(node,"nwProperties"));
		if(props==null){
			props = new Array();
		}


		// assign an array of properties that should be multi-valued
		if((arguments.length>2)&&(arguments[2])){
			var targ = arguments[2];
			if(targ.constructor == ("").constructor){
				targ = (new String(targ)).split("_||_");
			}
			for(var y in targ){
				mvald[targ[y]]=true;
			}
		}

		if(arguments.length>1){
			attrs = arguments[1];
			if(attrs.constructor != [].constructor){
				attrs = (new String(attrs)).split("_||_");
			}
			for(var x in attrs){
				x = this.stripMozIndex(attrs[x]);
				var ta = this.getAttr(node, x);
				if(ta != null){
					if(mvald[x]!=null){
						ta = (new String(ta)).split(";");
					}
					props[x]=ta;
				}
			}
		}

		// FIXME: is the next block right?
		var nwcn = "nwControlName";
		if(this.hasAttr(node, nwcn)){
			if(this.hasAttr(node,"nwValue")){
				props[nwcn]=this.getAttr(node, "nwValue");
			}else if(this.hasAttr(node,"name")){
				props[nwcn]=this.getAttr(node, "name");
			}else if(this.hasAttr(node,"nwId")){
				props[nwcn]=this.getAttr(node, "nwId");
			}else if(this.hasAttr(node,"id")){
				props[nwcn]=this.getAttr(node, "id");
			}
		}
		delete nwcn;

		// next we check child nodes to get node-style values
		if(node.hasChildNodes()){
			for(var tcnt=0; tcnt < (node.childNodes.length); tcnt++){
				var tnref = node.childNodes.item(tcnt);
				var snt = tnref.nodeType;
				// eliminates need for stripWS() call
				if( (snt <= 12)&&(snt != 3)&&(snt != 8) ){
					var tprop = this.getAttr(tnref, "nwProperty");
					if(tprop){
						var tval = this.getAttr(tnref, "nwValue");
						if(mvald[tprop]!=null){
							//mvald[tprop] = new Array();
							if(props[tprop] == null){
								props[tprop]=new Array();
							}
							if(tval != null){
								props[tprop][props[tprop].length]=tval;
							}else{
								props[tprop][props[tprop].length]=tnref.innerHTML;
							}
						}else{
							if(tval != null){
								props[tprop]=tval;
							}else{
								props[tprop]=tnref.innerHTML;
							}
						}
						delete tval;
					}
					delete tprop;
				}
				delete snt;
				delete tnref;
			}
		}
		// stdout("parsing required " + (((new Date())-start)/1000) + " seconds to complete");
		return props;
	}
}

__util__ = new NW_utility();
__utility_core__ = __util__;

// non-class helper functions
function NW_parseExpandoProperties(propertiesString){
	// we will return a multi-dimensional array representing the properties and
	// their arguments. First we split on semicolons.
	var rmWS = true;
	if(arguments.length>1){
		if(arguments[1]==false){
			rmWS = false;
		}
	}
	if(propertiesString){
		var whitespaceRegExp = new RegExp(" ","g");
		if(rmWS){
			propertiesString = propertiesString.replace(whitespaceRegExp, "");
		}
		var tempPropsArray = propertiesString.split(";");
		var propsArray = new Array();
		for(var key in tempPropsArray){
			if(tempPropsArray[key].length > 0){
				var elementArr = tempPropsArray[key].split(":");
				var attArr = elementArr[1].split(",");
				var spaceRE = new RegExp("\/","g");
				for (var i=(attArr.length-1); i>=0; i--){
					attArr[i] = attArr[i].replace(spaceRE, " ");
				}
				propsArray[elementArr[0]]=attArr;
			}
		}
		return propsArray;	
	}else{
		return null;
	}
}

function NW_stripWhiteSpaceNodes(dn){
	for(var node = (dn.childNodes.length-1); node >= 0; node--){
		var tnt = dn.childNodes[node].nodeType;
		if( (tnt == 8) || (tnt == 3) || (tnt == 12) ){
			dn.removeChild(dn.childNodes[node]);
		}
	}
	return dn;
}

//this function deals with a truly stupid bug in mozilla 0.9 and better.
// it seems that in using a "for in" loop, the index is returned in the format ["indexval"] and not the expected string, indexval.
function NW_stripMozIndexBug(indexVal) {
	var iVal = new String(indexVal);
	if(iVal.charAt()){
		if(iVal.charAt(0) == "["){ iVal = iVal.slice(2,-2); }
	}
	return iVal;
}

//class def, used to create elements that need event handlers in a way that hides
//the complexity of setting handlers in IE and Moz from th end user. Factory class.
function NW_element_maker_obj(){

	this.events = new Array();
	this.elementName = "";
	this.setType = function(elementName){
		this.elementName = elementName;
	}
	
	this.reset = new function(){
		this.events = new Array();
		this.elementName = "";
	}

	this.setEvent = function(eventName, eventStr){
		//check to see if we should replace, otherwise we assume append...
		eventName = eventName.toLowerCase();
		var replace = false;
		if((arguments.length>2)||(!this.events[eventName])){ replace = true; }
		if((this.events[eventName]==null)||(replace)){
			this.events[eventName]=eventStr;
		}else{
			this.events[eventName]+= " "+eventStr;
		}
	}
	
	this.getElement = function(){

		if((!__util__.isKonq)&&(!__util__.isMac)&&(!__util__.isOpera)&&(__util__.isIE)){
			//build the IEwin version
			var elemString = "<";
			elemString += this.elementName + " ";
			for(evt in this.events){
				elemString += evt+"=\""+this.events[evt]+"\" ";
			}
			elemString += ">";
			return document.createElement(elemString);
		}else{
			var elem = document.createElement(this.elementName);
			for(var evt in this.events){
				if(((!__util__.isKonq)&&(__util__.isIE))||(__util__.isOpera)){
					// set event handlers on Opera 7 and IE mac
					eval("elem."+evt+"=function(){"+this.events[evt]+"};");
				}else{
					// set the event handlers the standard way
					evt = NW_stripMozIndexBug(evt);
					elem.setAttribute(evt, this.events[evt]);
				}
			}
			return elem;
		}
	}

	//check for arguments to constructor
	if(arguments.length>0){
		this.setType(arguments[0]);
	}
	
}

// class NW_keyEvt, a class for describing a keystroke event catcher
// args are as follows:
// keyStr = "t" or whatever key you want to bind
// useShift = boolean, does the event require the shift key?
// useCtrl = boolean, does the event require the control key?
// useAlt = boolean, does the event require the alt key?

function NW_keyEvt(keyStr){
	// signal that fires when our key is called
	this.fireKey = function(){return true;}
	this.key = keyStr;
	this.useShift = false;
	this.useCtrl = false;
	this.useAlt = false;

	this.checkKey = function(key, shift, ctrl, alt){
		if((key == this.key)&&
		   (shift == this.useShift)&&
		   (ctrl == this.useCtrl)&&
		   (alt == this.useAlt)){
			if(arguments.length == 5){ __sig__.emit(this.fireKey); }
			return true;
		}else{
			return false;
		}
	}

	switch (arguments.length){
		case 1: break;
		case 2:
			this.useShift = arguments[1]; 
			break;
		case 3:
			this.useShift = arguments[1]; 
			this.useCtrl = arguments[2]; 
			break;
		case 4:
			this.useShift = arguments[1]; 
			this.useCtrl = arguments[2]; 
			this.useAlt = arguments[3]; 
		default: break;
	}
}

if(window.__scripts__){
	__scripts__.finalize(__config__.corePath+"utility_core.js");
}
