// $Id: widget_core.js,v 1.31 2003/02/02 09:56:26 russcoon Exp $
// Copyright (c) 2000-2003 Alex Russell
// Licensed under the Academic Free License version 1.1

// Filename:		widget_core.js
// Purpose:		declare and define some basic child elements of a componentObj
// Classes:		NW_widgetComponent
// Global Objects:	none
// Dependencies:	component_registry.js

with(__scripts__){
	provide(__config__.corePath+"widget_core.js");
	require(__config__.corePath+"component_registry.js");
}

function NW_widgetComponent(domNode){
	this.parent = NW_componentObj; // parent class
	this.parent(domNode);
	delete this.parent;

	// keystroke handling stuff. "feels" better to put it in the widget base def and not the
	// component def, as it's a bit over-broad for what's really a UI behavoiur and not (necessarialy?)
	// a general-purpose tool.
	this.keystrokes = new Array();

	// should be connected to delegating function
	this.fireKeys = function(key, sh, ctl, alt){
		for(var x in this.keystrokes){
			if(this.keystrokes[x].checkKey(key, sh, ctl, alt, true)){
				return true;
			}
		}
		return false;
	}

	this.checkKeys = function(key, sh, ctl, alt){
		for(var x in this.keystrokes){
			if(this.keystrokes[x].checkKey(key, sh, ctl, alt)){
				return this.keystrokes[x];
			}
		}
		return null;
	}

	this.setKeystroke = function(key, sh, ctl, alt, obj, fp){
		var kobj = this.checkKeys(key, sh, ctl, alt);
		if(kobj==null){
			kobj = new NW_keyEvt(key, sh, ctl, alt);
		}
		this.keystrokes[this.keystrokes.length]=kobj;
		__sig__.connect(kobj, kobj.fireKey, obj, fp);
	}

	this.unsetKeystroke = function(key, sh, ctl, alt, obj, fp){
		var kobj = this.checkKeys(key, sh, ctl, alt);
		if(kobj==null){
			__sig__.disconnect(kobj, kobj.fireKey, obj, fp);
		}
	}

}
NW_widgetComponent.prototype = new NW_componentObj; //set deep inheritance.

// a class for widgets that participate in forms
function NW_inputWidget(domNode){
	this.parent = NW_widgetComponent; // parent class
	this.parent(domNode);
	delete this.parent; // leave properties and methods in this namespace, but not the "parent" object itself

	this.dataType = null; // URI that should be set by subclasses
	this.editEnabled = true; // FIXME: different from spec, mail Mark about this.
	this.controlName = ""; // should be set by subclasses
	// FIXME: is this the best way to get a signal fired out of this?
	this.nameChanged = function(){ return true; }
	this.setName = function(cName){
		this.controlName = cName;
		__sig__.emit(this.nameChanged(this.controlName));
	}

	this.controlValue = null; // the current "value" of the form element
	// FIXME: is this the best way to get a signal fired out of this?
	this.valueChanged = function(){ return true; }
	this.setValue = function(cValue){
		this.controlValue= cValue;
		__sig__.emit(this.valueChanged(this.controlValue));
	}
	this.mandatory = false; // determines of a value (even a dummy space) is required for submittal of this form element.

	/* FIXME: in this class, we should define some method that will allow us to "attach" to a form, which should be in the  */

	this.attachToForm = function(){
		if((this.domNode.parentNode)&&(this.domNode.parentNode != this.domNode)){
			/* 
			create a simple, invisible, form element to add to the form. 
			The value of this element is what will "really" be updated 
			when we change the display value of this widget.
			*/
			// FIXME: finish this method when more awake
		}
	}

	this.toggleEditEnabled = function(){
		if(this.editEnabled == true){
			__sig__.emit(this.setEditDisabled);
		}else{
			__sig__.emit(this.setEditEnabled);
		}
	}

	// subclasses should connect to these signals and act upon them
	this.setEditEnabled = function(){ this.editEnabled = true; }; // signal
	this.setEditDisabled = function(){ this.editEnabled = false; }; // signal
	// these semantics allow us to ensure that the function will fire even if it doesn't have any slots
	__sig__.addBareSignal(this, this.setEditEnabled);
	__sig__.addBareSignal(this, this.setEditDisabled);
}
NW_inputWidget.prototype = new NW_widgetComponent;

// a widget registry to allow the registered widget types to be located
__widgets__ = new Object();
__widgets__.regArr = new Array();
__widgets__.register = function(widgetType, inlineConstuctor, normalConstructor){
	// we don't ever expect to unregister a widget.
	this.regArr[widgetType] = new NW_widgetDescriptorObject(widgetType, inlineConstuctor, normalConstructor);
}

__widgets__.isWidgetComp = function(widgetName){
	if(this.regArr[widgetName] != null){
		return true;
	}else{
		return false;
	}	
}

__widgets__.runConstructor = function(widgetName, args){
	if(this.isWidgetComp(widgetName)){
		var widgetObj = eval(this.regArr[widgetName].constructor + "(" + args + ")");
		return widgetObj;
	}else{
		return null;
	}
}

__widgets__.runInlineConstructor = function(wName, domNode){
	var widgetName = wName;
	if(this.isWidgetComp(widgetName)){
		//workaround for funky Moz 0.9.x bug, horrible hack...
		widgetName = __util__.stripMozIndex(widgetName);
		if(this.regArr[widgetName]){
			// FIXME: we should pass/call function pointers instead
			var widgetObj = eval(this.regArr[widgetName].inlineConstructor + "(domNode);");
			return widgetObj;
		}else{
			return null;
		}
	}else{
		stdout("widget type " + widgetName + " not registered");
		return null;
	}
}

// data structure for defining widgets internally
function NW_widgetDescriptorObject(name, inlineConst, regConst){
	this.name = name;
	this.constructor = regConst;
	this.inlineConstructor = inlineConst;
}

__NW_widgets__ = __widgets__;

__scripts__.finalize(__config__.corePath+"widget_core.js");
