// $Id: button_widget.js,v 1.42 2003/01/31 03:01:11 russcoon Exp $
// Copyright (c) 2000-2002 Alex Russell
// Licensed under the Academic Free License version 1.1

// Filename:		button_widget.js
// Purpose:		defines the button widget
// Classes:		NW_WIDGET_button
// Global Objects:	none
// Dependencies:	widget_core.js, utility_core.js,
//			theme_core.js, environ_core.js, component_registry.js

/*

	CSS classes for this widget:

		x Button-background-div
		x Button-background-div-hover
		x Button-background-div-pressed
		x Button-content-div
		x Button-content-div-dissabled
		x Button-content-img

*/

with(__scripts__){
	provide(__config__.widgetPath+"button_widget.js");
	require(__config__.corePath+"widget_core.js");
	require(__config__.corePath+"utility_core.js");
	require(__config__.corePath+"theme_core.js");
	require(__config__.corePath+"environ_core.js");
}

function NW_WIDGET_button(domNode, txtNode, imgNode){
	//set inheritance
	this.parent = NW_widgetComponent;
	this.parent(domNode);
	delete this.parent;
	//general NW component attributes
	this.isTopLevelComp = false;
	this.isDraggable = false;
	this.componentType = "button";
	this.resize = null;

	this.focus = function(){
		if(this.domNode.focus){ // Moz supports focus on any element
			this.domNode.focus();
		}
	}

	// button specific settings and methods
	// since we'll need to set most of the properties of a button after
	// the button is instianted, provide methods to do this.
	this.buttonTxtNode = txtNode;
	this.buttonImgNode = imgNode;

	this.setText = function(text){ // change the current value
		this.buttonTxtNode.innerHTML = text; }

	// not rocket science...
	this.setWidth = function(width){
		this.domNode.style.width = parseInt(width)+"px"; }

	this.setHeight = function(height){
		this.domNode.style.height = parseInt(height)+"px"; }

	// text strings to be eval'd
	this.onHighlight = "";
	this.onClick = "";
	this.onLeave = "";
	this.bounceBack = true;
	this.isActive = true;

	this.setOnHighlight = function(codeStr){
		this.onHighlight += codeStr; 
	}

	this.doOnHighlight = function(){
		if(this.isActive==true){
			__util__.setClass(this.domNode, "Button-background-div-hover");
			if(__config__.applyThemedStyles){
				with(this.domNode.style){
					borderWidth = "2px 2px 2px 2px";
					padding = "1px 1px 1px 1px";
					borderStyle = "outset";
					borderColor = __theme__.getThemeAttr("menu_bgcolor");
				}
			}
		}
		eval(this.onHighlight);
	}
	__sig__.addBareSig(this, this.doOnHighlight);

	this.setOnClick = function(codeStr){
		this.onClick += codeStr; 
	}

	this.doOnClick = function(event){
		if(this.isActive==true){
			__util__.setClass(this.domNode, "Button-background-div-pressed");
			if(__config__.applyThemedStyles){
				this.domNode.style.borderStyle = "inset";
			}
			eval(this.onClick);
			if((this.onClick!="")||(this.bounceBack==false)){ // make the button "bounce back" after it's been clicked
				setTimeout('if(__config__.applyThemedStyles){__components__.getComponent('+this.globIdNum+').domNode.style.borderStyle="outset";}; __util__.setClass(__components__.getComponent('+this.globIdNum+').domNode,"Button-background-div");', 100)
			}
			return true;
		}else{return false;}
	}
	__sig__.addBareSig(this, this.doOnClick);


	this.setOnLeave = function(codeStr){
		this.onLeave += codeStr; 
	}

	this.doOnLeave = function(){
		if(this.isActive==true){
			__util__.setClass(this.domNode, "Button-background-div");
			if(__config__.applyThemedStyles){
				with(this.domNode.style){
					borderWidth = "1px 1px 1px 1px";
					padding = "2px 2px 2px 2px";
					borderStyle = "outset";
				}
			}
		}
		eval(this.onLeave);
	}
	__sig__.addBareSig(this, this.doOnLeave);

	// set state
	this.setInactive = function(){
		if(this.isActive == false){return true;}
		this.isActive = false;
		__util__.setClass(this.domNode, "Button-content-div-dissabled");
		if(__config__.applyThemedStyles){
			__theme__.modifyTheme("color:head_bar_bgcolor;", this.domNode);
			__theme__.applyTheme(this.domNode.parentNode);
		}
	}
	
	this.setActive = function(){
		if(this.isActive == true){return true;}
		this.isActive = true;
		__util__.setClass(this.domNode, "Button-content-div");
		if(__config__.applyThemedStyles){
			__theme__.modifyTheme("color:menu_font_color;", this.domNode);
			__theme__.applyTheme(this.domNode.parentNode);
		}
	}
	
	//destructor methods
	// **** should these be moved to the general component def? ****
	this.destructorCode = "this.remove();";
	this.setDestructor = function(code){
		this.destructorCode = code; 
	}
	
	this.destructor = function(){
		eval(this.destructorCode); 
	}
}
NW_WIDGET_button.prototype = new NW_widgetComponent; //deep inhertiance support
__widgets__.register("button", "NW_WIDGET_INLINE_FACTORY_button", "NW_WIDGET_FACTORY_button");

// to do: add "message" field to allow users to set some text durring the progress bar's load
function NW_WIDGET_FACTORY_button(buttonTxt, pwidth, pheight){
	var bdmaker = new NW_element_maker_obj("div");
	// provide hooks to extensible event handlers
	var gfcstr = "__components__.getComponentFromNode(this)";
	with(bdmaker){
		setEvent("onMouseDown","__sig__.emit("+gfcstr+".doOnClick, event);");
		setEvent("onMouseOver","__sig__.emit("+gfcstr+".doOnHighlight);");
		setEvent("onMouseOut","__sig__.emit("+gfcstr+".doOnLeave);");
	}
	var BDomNode = bdmaker.getElement();
	BDomNode.setAttribute("align","center");
	__util__.setClass(BDomNode, "Button-background-div");
	with(BDomNode.style){
		width = parseInt(pwidth) + "px";
		height = (parseInt(pheight)-4) + "px";
		position = "relative";
		visibility = "visible";
		if((__env__.is_IE)&&(!__env__.is_konq)){
			cursor = "hand";
		}else{
			cursor = "pointer";
		}
		letterSpacing = "1px";
		borderWidth = "1px 1px 1px 1px";
		padding = "2px 2px 2px 2px";
		borderStyle = "outset";
		//if(__env__.is_IE&&!__env__.is_mac){
		//    display = "inline";
		//}
		if(!__env__.is_IE){MozBorderRadius="6px 6px 6px 6px";}
	}
	with(__theme__){
		setStyleByAttr(BDomNode, "main_bgcolor", "backgroundColor");
		setStyleByAttr(BDomNode, "menu_border_color", "borderColor");
		setStyleByAttr(BDomNode, "menu_font_size", "fontSize");
		setStyleByAttr(BDomNode, "menu_font_color", "color");
	}
	
	var msgNode = null;
	var msgmaker = new NW_element_maker_obj("div");
	msgNode =  msgmaker.getElement();
	__util__.setClass(msgNode, "Button-content-div");
	if(arguments.length>3){//handle image URL if passed
		var bImg = document.createElement("img");
		__util__.setClass(bImg, "Button-content-img");
		bImg.src = arguments[3];
		bImg.align = "left";
		if(__env__.is_konq){
			bImg.style.paddingLeft = "10px";
			bImg.style.paddingTop = "8px";
		}
		msgNode.appendChild(bImg);
	}
	msgNode.appendChild(document.createTextNode(buttonTxt));
	BDomNode.appendChild(msgNode);

	var barObj = new NW_WIDGET_button(BDomNode, msgNode, null);
	BDomNode.setAttribute("NW_compID", barObj.globIdNum);
	BDomNode.id = barObj.globIdNum;
	return barObj;
}


function NW_WIDGET_INLINE_FACTORY_button(domNode){
	var attrArr = ['text', 'image', 'width', 'height', 'onHighlight', 'doOnClick', 'onLeave', 'styleList'];
	var props = __util__.parseNWProps(domNode, attrArr);
	var tButton = null;
	var bTxt = props["text"];
	var bImg = null;
	if(props["image"]){
		bImg = props["image"];
	}
	var bW = props["width"];
	var bH = props["height"];
	if((bImg != null)&&(bImg != "")){
		tButton = NW_WIDGET_FACTORY_button(bTxt, bW, bH, bImg);
	}else{
		tButton = NW_WIDGET_FACTORY_button(bTxt, bW, bH);
	}
	if(props["onHighlight"]){
		tButton.setOnHighlight(props["onHighlight"]);
	}
	if(props["doOnClick"]){
		tButton.setOnClick(props["doOnClick"]);
	}
	if(props["onLeave"]){
		tButton.setOnLeave(props["onLeave"]);
	}
	if(props["styleList"]){
		__util__.setNodeStyle(props["styleList"], tButton.domNode);
	}
	if(props["tooltip"]){
		tButton.domNode.setAttribute("tooltip", props["tooltip"]);
		tButton.buttonTxtNode.setAttribute("tooltip", props["tooltip"]);
	}
	return tButton;
}

__scripts__.finalize(__config__.widgetPath+"button_widget.js");
