// $Id: gen_data_table_widget.js,v 1.86 2003/01/27 00:38:09 russcoon Exp $
// Copyright (c) 2000-2002 Alex Russell
// Licensed under the Academic Free License version 1.1

// Filename:		gen_data_table_widget.js
// Purpose:		displays data in resizeable columns with headers
// Classes:		NW_WIDGET_gen_data_table
// Global Objects:	none
// Dependencies:	widget_core.js, theme_core.js, environ_core.js, component_registry.js, utility_core.js

// FIXME: need to "signalify" all of the lookup code
// TODO: investigate a "truncateValues" option. Can we do ellipsis generation well?
// TODO: refactor sorting to make use of sortOrder property
// TODO: Provide eval()-able way to provide own comparator func?

/*
we implement the following CSS classes, although the node names given MAY NOT REFLECT THE INTERNAL STRUCTURE OF THE WIDGET:
	x Table-top-table
	x Table-heading-thead
	x Table-heading-tr
	x Table-heading-value-td
	x Table-data-tbody
	x Table-value-td
	x Table-dataRow-tr
*/

with(__scripts__){
	var wcp = __config__.corePath;
	provide(__config__.widgetPath+"gen_data_table_widget.js");
	require(wcp+"utility_core.js");
	require(wcp+"widget_core.js");
	require(wcp+"theme_core.js");
	require(wcp+"environ_core.js");
	require(wcp+"component_registry.js");
	require(__config__.widgetPath+"split_pane_widget.js");
	delete wcp;
}

function NW_WIDGET_gen_data_table(domNode){
 	this.parent = NW_widgetComponent;
	this.parent(domNode);
	delete this.parent;
	this.domNode = domNode;
	this.isTopLevelComp = false;
	this.isDraggable = false;
	this.componentType = "gen_data_table";

	this.columns = new Array();
	this.rows = new Array();
	this.row_keys = new Array();
	this.headerRowNode = null; //the DOM node corresponding to the header bar
	this.displayTableNode = null;
	this.displaySpacingRow = null;
	this.dataContainer = null;
	this.truncateValues = false;
	this.canResize = false;
	this.hasResized = false;
	this.canDeleteRow = true;
	this.rowHeight = "18px";
	this.highlightedRow = null;
	this.panes = null;
	this.sortedColumn = null;
	this.sortImgNode = null;
	this.sortEnabled = true;
	this.sortEmptyLast = true;
	this.incomparableOrder = "last";
	this.sortReversed = false;
	this.upImgSrc = __config__.imgPath+"up_sort.png";
	this.downImgSrc = __config__.imgPath+"down_sort.png";

	this.sorts = __util__.sorts;

	this.doOnPlacement = function(){
		this.panes.doOnPlacement();
		this.fixSizing();
	}

	this.fixSizing = function(){
		if(__env__.is_IE){
			var hcn = this.panes.pane1obj.contentNode;
			var dcn = this.panes.pane2obj.contentNode;
			hcn.firstChild.style.width = (dcn.clientWidth)+"px";
			dcn.firstChild.style.width = (dcn.clientWidth)+"px";
		}
		if(this.displayTableNode.firstChild){
			for(var x = 0; x < this.headerRowNode.childNodes.length-1; x++){
				if(this.displayTableNode.childNodes.length > 1){
					var cw = this.displayTableNode.childNodes[1].childNodes[x].offsetWidth;
				}else{
					var cw = this.displayTableNode.firstChild.childNodes[x].offsetWidth;
				}
				if(__env__.is_IE){
					cw -= 2;
				}
				if(cw<4){ cw = 4; }
				var colObj = this.columns[x];
				with(colObj.colHeaderNode){
					style.width = cw+"px";
					// need to make sure this isn't a sort image!
					if(firstChild!=this.sortImgNode){
						var cchild = firstChild;
					}else{
						var cchild = childNodes[1];
					}
					// if(((this.truncateValues)&&(cchild.offsetWidth>cw))||(cchild.offsetWidth>cw)){
					if(this.truncateValues){
						cchild.style.overflow = "hidden";
						cchild.style.position = "absolute";
						cchild.style.top = "0px";
						if(__env__.is_IE){
							cchild.style.textOverflow = "ellipsis";
						}
					}
					cchild.style.width = (cw-4)+"px";
					setAttribute("width", cw);
				}
			}
		}
	}
	__sig__.addBareSig(this, this.fixSizing);

	this.lookupBaseStr  = "__components__.getComponentFromNode(this).getParentOfType('gen_data_table')";
	//class for defining the column data type
	function columnDef(colName, colNode, colDataNode, colIndex){
		this.colName = colName;
		this.colHeaderNode = colNode; //reference to the node that corresponds to the display of the name of the column
		this.colDataNode = colDataNode;//reference to node used in re-sizing the data columns, should be empty, 1px high TD
		this.index = colIndex;
		this.width = null;
	}

	this.initResize = function(domNode){
		__env__.disableDragAndSelect();
		this.selectedColumn = this.lookupColumn(domNode);
		this.canResize = true;
		return true;
	}
	
	this.cancelResize = function(){
		__env__.enableDragAndSelect();
		this.domNode.style.width = this.domNode.style.width;
		if(this.selectedColumn){
			var ci = this.getColumnIndex(this.selectedColumn);
			// this.resizeColumn(ci, this.selectedColumn.width);
		}
		this.selectedColumn = null;
		this.canResize = false;
		return true;
	}

	this.getColumnIndex = function(colObj){
		var clen = this.columns.length;
		for(var x = 0; x < clen; x++){
			if(colObj == this.columns[x]){ return x; }
		}
	}

	this.lookupColumnIndex = function(domNode){
		for(var x = 0; x < this.columns.length; x++){
			if(this.columns[x].colHeaderNode == domNode){
				return x;
			}
		}
		return 0;
	}

	this.lookupColumn = function(domNode){
		for(x in this.columns){
			if(this.columns[x].colHeaderNode == domNode){
				return this.columns[x];
			}
		}
		return this.columns[0];
	}

	this.lookupRow = function(domNode){
		//FIXME: need to finish this method
		/*
		for(x in this.rows){
			if(this.columns[x].colHeaderNode == domNode){
				return this.columns[x];
			}
		}
		return this.columns[0];
		*/
		return this.rows[0];
	}

	this.stripForSort = function(str){
		// get rid of HTML
		str = str.replace(/<.*?>/gm,"");
		// remove leading spaces
		str = str.replace(/^(&nbsp\;|&#160\;|\s|\n|\r)*/gm,"");
		// remove trailing spaces
		str = str.replace(/((&nbsp\;|&#160\;|\s|\n|\r)*)?$/gm,"");
		// replace tabs and other whitespace with a single space
		str = str.replace(/(\t|&nbsp\;|&#160\;|\s|\r|\n)+/gm," ");
		return str;

	}

	// FIXME: need to implemenent ordered sorting here!
	this.sortColumn = function(domNode){
		if(!this.sortEnabled){
			return 1;
		}
		// find the column we want to sort
		var col = this.lookupColumnIndex(domNode);
		// FIXME: make sure the column header is wide enough
		// this.resizeColumn(col, __util__.getCSVal(this.headerRowNode.childNodes[col], 'width'));
		var data = new Array();
		var blanks = new Array();
		var key_data = new Array();
		// this first step here is to get associatve arrays of keys and values
		for(x in this.rows){
			// FIXME: need to make some effort here to get rid of potential HTML in the strings before splitting
			var rtref = this.rows[x]["tr"];
			if((col==null)||(!rtref.childNodes)||(col>=rtref.childNodes.length)||(!rtref.childNodes[col])){
				var ihtml = "";
			}else{
				var ihtml = this.stripForSort(rtref.childNodes[col].innerHTML||"");
			}
			var strippedIhtml = ihtml.replace(/(&nbsp\;|&#160\;|\n|\r|\s)*/gm,"");
			if((strippedIhtml=="")||(ihtml=="&nbsp;")||((ihtml.length==1)&&(ihtml[0]==" "))){
				blanks[blanks.length]=ihtml;
			}else{
				data[data.length]=ihtml;
			}
			if(key_data[ihtml]==null){
				// create a new bucket
				key_data[ihtml] = new Array(this.rows[x]["key"], x);
			}else{
				// add another index to the bucket
				key_data[ihtml][key_data[ihtml].length]=x;
			}
		}

		// FIXME: need to add support for custom comparitors here
		var sortType = __util__.getAttr(domNode,"sortType");
		if((sortType!=null)&&(this.sorts[sortType])){
			data.sort(this.sorts[sortType]);
		}else{
			data.sort();
		}

		if((col == this.sortedColumn)&&(this.sortReversed==false)){
			this.sortReversed = true;
		}else{
			this.sortReversed = false;
			data.reverse();
		}

		if(this.incomparableOrder == "last"){
			data = data.concat(blanks);
		}else if(this.incomparableOrder == "first"){
			data = blanks.concat(data);
		}else if(this.incomparableOrder == "highest"){
			// stdout('highest!');
			if(this.sortReversed){
				// blanks = blanks.reverse();
				data = data.concat(blanks);
			}else{
				data = blanks.concat(data);
			}
		}else if(this.incomparableOrder == "lowest"){
			if(this.sortReversed){
				data = blanks.concat(data);
			}else{
				data = data.concat(blanks);
			}
		}else{
			data = data.concat(blanks);
		}

		if(this.sortImgNode != null){
			// this.sortedColumn.removeChild(this.sortImgNode);
			this.columns[this.sortedColumn].colHeaderNode.removeChild(this.sortImgNode);
			this.sortImgNode = null;
		}

		// now re-order our display to reflect the new arrangement
		var rcount = 0;
		for(x in data){
			var kdr = key_data[data[x]];
			var rows_index = kdr[kdr.length-1];
			var trref = this.displayTableNode.removeChild(this.rows[rows_index]["tr"]);
			// this.displayTableNode.insertBefore(trref, this.displayTableNode.lastChild);
			this.displayTableNode.appendChild(trref);
			// there can be multilple identical values with the same key, so 
			// we remove "used" values so we garuntee a unique key (given uniuqe
			// keys as in input constraint)
			if(kdr.length>1){
				// chop the array
				kdr.length = kdr.length-1;
			}
			rcount++;
		}
		this.sortedColumn = col;

		var simgMkr = new NW_element_maker_obj("img");
		simgMkr.setEvent("onClick", this.lookupBaseStr+".sortColumn(this.parentNode);");
		this.sortImgNode = simgMkr.getElement();
		with(this.sortImgNode){
			// style.display = "inline";
			align = "right";
			style.paddingTop = "3px";
			style.paddingLeft = "3px";
			style.paddingRight = "6px";
			style.width = "10px";
			style.height = "10px";
			style.left = "10px";
			style.top = "10px";
			if(this.sortReversed){
				src = this.upImgSrc;
			}else{
				src = this.downImgSrc;
			}
		}

		// this.columns[this.sortedColumn].colHeaderNode.appendChild(this.sortImgNode);
		with(this.columns[this.sortedColumn].colHeaderNode){
			insertBefore(this.sortImgNode, firstChild);
			// appendChild(sortImgNode);
		}
		delete data;
		delete key_data;
		return true;
	}


	this.addColumn = function(colName, cwidth){
		var cngen = new NW_element_maker_obj("td");
		var colNode = cngen.getElement();
		__util__.setClass(colNode, "Table-heading-value-td");
		var ctngen = new NW_element_maker_obj("div");
		ctngen.setEvent("onClick",this.lookupBaseStr+".sortColumn(this.parentNode);");
		var colTitleNode = ctngen.getElement();
		if(!__env__.is_IE){
			colNode.appendChild(colTitleNode);
		}
		// FIXME: should we make the title node have bold text as well?
		// inlining for speed
		with(colTitleNode.style){
			whiteSpace = "nowrap"; 
			textDecoration = "underline"; 
			margin = "0px"; 
			borderRight = "1px solid "+__theme__.getAttr("menu_bgcolor"); 
			paddingRight = "3px"; 
			height = "18px";
		}
		colTitleNode.innerHTML = colName;

		// this.headerRowNode.insertBefore(colNode, this.headerRowNode.lastChild);
		this.headerRowNode.appendChild(colNode);
		with(__theme__){
			setStyleByAttr(colNode, "main_font_size", "fontSize");
			setStyleByAttr(colNode, "menu_hl_font_color", "color");
			setStyleByAttr(colNode, "menu_hl_bgcolor", "backgroundColor");
			// setStyleByAttr(colNode, "menu_hl_color", "borderColor");
		}

		with(colNode.style){
			if(cwidth){
				width = cwidth + "px";
			}
			height = this.rowHeight;
			if(!__env__.is_IE){
				position = "relative";
			}
			borderBottom="1px solid gray";
			overflow = "hidden";
		}

		colNode.setAttribute("height",parseInt(this.rowHeight));
		if(cwidth){
			colNode.setAttribute("width",cwidth);
		}
		if((arguments.length>2)&&(arguments[2]!=null)){
			colNode.setAttribute("sortType",arguments[2]);
		}
		if((arguments.length>3)&&(arguments[3]!=null)){
			colNode.setAttribute("sortOrder",arguments[3]);
		}
		if((arguments.length>4)&&(arguments[4]!=null)){
			__util__.setClass(colNode, "Table-heading-value-td "+arguments[4]);
		}
		if(__env__.is_IE){
			colNode.appendChild(colTitleNode);
		}

		//this cell will set the spacing for row in which it sits
		var colDNode = document.createElement("td");
		this.displaySpacingRow.appendChild(colDNode);
		
		with(colDNode.style){
			height = "1px";
			if(cwidth){
				width = cwidth + "px";
			}
			overflow = "hidden";
			position = "absolute";
		}
		with(colDNode){
			setAttribute("height","1");
			if(cwidth){
				setAttribute("width",cwidth);
			}
		}


		colObj = new columnDef(colName, colNode, colDNode, this.columns.length);
		this.columns[this.columns.length]=colObj;
		colObj.width = cwidth;

		if(__env__.is_IE){
			// hack to get IE to correctly size the column we just added
			this.selectedColumn = colObj;
			this.cancelResize();
		}

		//need to add logic here to adjst the current rows, perhaps polling the data source for new data?

	}

	// define signals
	this.sigChangeSelectedRow = function(key, domRef){ return true; }
	this.sigRemoveRow = function(key, domRef){ return true; }
	this.sigAddRow = function(key, domRef){ return true; }
	
	this.dispRowCount = 0;
	this.displayedRows = new Array();
	this.selectedRowNode = null;
	this.setSelectedRow = function(domNode){
		if(this.selectedRowNode){
			__theme__.setStyleByAttr(this.selectedRowNode, "menu_bgcolor", "backgroundColor");
		}

		while(domNode.nodeName.toLowerCase() != "tr"){domNode=domNode.parentNode;}
		this.selectedRowNode = domNode;
		__theme__.setStyleByAttr(this.selectedRowNode, "content_bgcolor", "backgroundColor");
		var tkey = null;
		for(x in this.rows){
			if(this.rows[x]["tr"]==this.selectedRowNode){
				tkey = this.rows[x]["key"];
				break;
			}
		}

		__sig__.emit(this.sigChangeSelectedRow, tkey, this.selectedRowNode);

	}

	// FIXME: these functions need to deal better with sorted columns where the first child is not a collapsed node

	this.selectPrevious = function(){
		if(this.selectedRowNode==null){
			if(this.displayTableNode.firstChild!=this.displayTableNode.lastChild){
				this.setSelectedRow(this.displayTableNode.lastChild.previousSibling);
				if((!__env__.is_IE)&&(this.selectedRowNode.scrollIntoView)){
					this.selectedRowNode.nextSibling.scrollIntoView(false);
				}
			}
		}else{
			with(this.selectedRowNode){
				if((previousSibling)&&(previousSibling!=this.displayTableNode.lastChild)){
					this.setSelectedRow(previousSibling);
					if((!__env__.is_IE)&&(scrollIntoView)){
						previousSibling.scrollIntoView(true);
					}
				}
			}
		}
	}

	this.selectNext = function(){
		if(this.selectedRowNode==null){
			if(this.displayTableNode.firstChild!=this.displayTableNode.lastChild){
				this.setSelectedRow(this.displayTableNode.firstChild);
				if((!__env__.is_IE)&&(this.selectedRowNode.scrollIntoView)){
					this.selectedRowNode.nextSibling.scrollIntoView(false);
				}
			}
		}else{
			with(this.selectedRowNode){
				if((nextSibling)&&(nextSibling!=this.displayTableNode.lastChild)){
					this.setSelectedRow(nextSibling);
					if((!__env__.is_IE)&&(scrollIntoView)){
						nextSibling.scrollIntoView(false);
					}
				}
			}
		}
	}

	this.removeRow = function(){
		if((this.selectedRowNode)&&(this.canDeleteRow)){
			var tkey = null; var tindex = null;
			for(x in this.rows){
				if(this.rows[x]["tr"]==this.selectedRowNode){
					tkey = this.rows[x]["key"];
					tindex = x;
					break;
				}
			}
			__sig__.emit(this.sigRemoveRow, tkey, this.selectedRowNode);
			this.selectedRowNode.parentNode.removeChild(this.selectedRowNode);
			delete this.rows[tindex];
			this.selectedRowNode = null;
		}
	}

	this.addRow = function(key, rowArr){
		var rowCrtr = new NW_element_maker_obj("tr");
		rowCrtr.setEvent("onMouseDown",this.lookupBaseStr+".setSelectedRow(this);");
		var rowElem = rowCrtr.getElement();
		// rowElem.setAttribute("class","Table-dataRow-tr");
		__util__.setClass(rowElem, "Table-dataRow-tr");
		// this.displayTableNode.insertBefore(rowElem, this.displayTableNode.firstChild);
		this.displayTableNode.insertBefore(rowElem, this.displayTableNode.lastChild);
		__theme__.setStyleByAttr(rowElem, "main_font_size", "fontSize");
		__theme__.setStyleByAttr(rowElem, "menu_bgcolor", "backgroundColor");
		rowElem.setAttribute("valign", "top");

		// add space for another row
		var tlen = this.rows.length;
		this.rows[tlen] = new Array();
		rowElem.setAttribute("NW_table_key", key);
		this.rows[tlen]["tr"]=rowElem;
		this.rows[tlen]["key"]=key;

		for (var i = 0; i < this.columns.length; i++){
			var ttd = document.createElement("td");
			if((arguments.length>2)&&(arguments[2]!=null)){
				__util__.setClass(ttd, "Table-value-td "+arguments[2][i]);
			}else{
				__util__.setClass(ttd, "Table-value-td");
			}
			rowElem.appendChild(ttd);
			// ttd.setAttribute("nowrap", "nowrap");
			with(ttd.style){
				position="relative";
				borderRight="1px solid gray";
				paddingLeft="3px";
			}
			__theme__.setStyleByAttr(ttd, "main_border_color", "borderColor");
			// ttd.appendChild(document.createTextNode(rowArr[i]));

			if(this.truncateValues){
				var ttddiv = document.createElement("div");
				with(ttddiv.style){
					position = "relative";
					overflow = "hidden";
					cssFloat = "left";
				}
				with(ttddiv.style){
					if(this.columns[i].width){
						width = this.columns[i].width + "px";
						ttd.setAttribute("width", parseInt(width));
					}
					if(__env__.is_IE){
						textOverflow = "ellipsis";
					}
					whiteSpace = "nowrap";
				}
				ttd.appendChild(ttddiv);
				// using innerHTML to satisfy some of Nitin's requirements
				ttddiv.innerHTML = rowArr[i];
			}else{
				ttd.innerHTML = rowArr[i];
			}

			this.rows[tlen][i]=ttd;
		}
		// tell the rest of the world that the row was successfully added
		__sig__.emit(this.sigAddRow, key, this.rows[tlen]["tr"]);
		return true;
	}

}
NW_WIDGET_gen_data_table.prototype = new NW_widgetComponent; //deep inhertiance support
__widgets__.register("gen_data_table", "NW_WIDGET_FACTORY_inline_gen_data_table", "NW_WIDGET_FACTORY_gen_data_table");


function NW_WIDGET_FACTORY_gen_data_table(){
	// we rely on the split pane widget to create our table container for us
	var cmpstr = "__components__.getComponentFromNode(this).getParentOfType('gen_data_table')";
	var mdiv = document.createElement("div");
	with(mdiv.style){
		position = "relative";
	}
	__util__.setClass(mdiv, "Table-top-table");
	var paneObj = new NW_WIDGET_FACTORY_split_pane(false, "18", false, 3);
	with(paneObj){
		pane1obj.domNode.style.width="100%";
		pane1obj.contentNode.style.width="100%";
		pane1obj.domNode.style.border="0px";
		__util__.setClass(pane1obj.contentNode, "Table-heading-thead");
		pane2obj.domNode.style.width="100%";
		pane2obj.domNode.style.border="0px";
		pane2obj.contentNode.style.width="100%";
	}
	mdiv.appendChild(paneObj.domNode);
	if(__env__.is_IE){
		paneObj.pane1obj.contentNode.style.overflow = "hidden";
	}

	//inner, column display table
	var htbl = document.createElement("table");
	__theme__.setStyleByAttr(htbl, "menu_hl_bgcolor", "backgroundColor");
	__theme__.setStyleByAttr(paneObj.pane1obj.contentNode, "menu_hl_bgcolor", "backgroundColor");
	paneObj.pane1obj.contentNode.appendChild(htbl);
	with(htbl){
		setAttribute("cellpadding","0");
		setAttribute("cellspacing","0");
		setAttribute("border","0");
		setAttribute("width","100%");
	}
	with(htbl.style){
		width = "100%";
		left = "0px";
		top = "0px";
		border = "0px";
		margin = "0px";
		padding = "0px";
	}
	var htbdy = document.createElement("tbody");
	htbl.appendChild(htbdy);

	var hctrgen = new NW_element_maker_obj("tr");
	var hctr = hctrgen.getElement();
	__util__.setClass(hctr, "Table-heading-tr");
	htbdy.appendChild(hctr);

	__theme__.setStyleByAttr(hctr, "menu_hl_bgcolor", "backgroundColor");
	with(hctr){
		style.width = "100%";
		style.overflow = "hidden";
		style.height = "18px";
		setAttribute("width","100%");
		setAttribute("height","18");
	}

	//inner, data display table
	var dtblgen = new NW_element_maker_obj("table");
	// dtblgen.setEvent("onMouseOver",cmpstr+".panes.fixBottomPane();");
	var dtbl = dtblgen.getElement();
	__util__.setClass(dtbl, "Table-data-tbody");
	paneObj.pane2obj.contentNode.appendChild(dtbl);

	with(dtbl){
		setAttribute("cellpadding","0");
		setAttribute("cellspacing","0");
		setAttribute("border","0");
	}
	with(dtbl.style){
		width = "100%";
		if(__env__.is_IE){
			width = "98%";
		}
		left = "0px";
		top = "0px";
	}

	
	var dtbdy = document.createElement("tbody");
	dtbl.appendChild(dtbdy);

	//this row will hold the "spacing" cells that will determine the width of each of the columns
	var dctr = document.createElement("tr");
	dtbdy.appendChild(dctr);
	dctr.setAttribute("width","100%");
	dctr.style.height = "1px";
	if(__env__.is_IE){
		dctr.style.display = "none";
	}

	//make a widget object and link all this stuff up.
	var dto = new NW_WIDGET_gen_data_table(mdiv);
	mdiv.setAttribute("NW_compID", dto.globIdNum);
	mdiv.id = dto.globIdNum;

	with(dto){
		headerRowNode = hctr;
		displayTableNode = dtbdy;
		dataContainer = paneObj.pane2obj.contentNode;
		displaySpacingRow = dctr;
	}

	dto.panes = paneObj;

	// assign keystroke handlers
	dto.setKeystroke('up', false, false, false, dto, dto.selectPrevious);
	dto.setKeystroke('down', false, false, false, dto, dto.selectNext);
	dto.setKeystroke('del', false, false, false, dto, dto.removeRow);

	return dto;
}

function NW_WIDGET_FACTORY_inline_gen_data_table(domNode){
	// FIXME: need to add support for "standard" attributes to this constructor
	var attrArr = ['sortOrder', 'sortEmptyLast', 'sortBlankAs','sortComparator', 
			'columnName', 'upArrowImage', 'downArrowImage', 
			'sortEnabled', 'sortOnLoad', 'width', 'height', 
			'key', 'cell', 'parseChildren', 'truncate', 'class',
			'canDelete'];
	var mVald = ['cell', 'class'];
	var parseObj = __util__.parseNWComponents(domNode, attrArr, mVald);
	// call the "sort only" constructor
	var widgetObj = NW_WIDGET_FACTORY_gen_data_table();

	if(parseObj["upArrowImage"]){
		widgetObj.upImgSrc = parseObj["upArrowImage"];
	}
	if(parseObj["downArrowImage"]){
		widgetObj.downImgSrc = parseObj["downArrowImage"];
	}
	if(parseObj["sortBlankAs"]){
		widgetObj.incomparableOrder= parseObj["sortBlankAs"];
	}

	if((parseObj["sortEmptyLast"])&&(parseObj["sortEmptyLast"]=="false")){
		widgetObj.sortEmptyLast = false;
	}

	if((parseObj["canDelete"])&&(parseObj["canDelete"]=="false")){
		widgetObj.canDeleteRow = false;
	}

	widgetObj.truncateValues = false;
	if((parseObj["truncate"])&&(parseObj["truncate"]=="true")){
		widgetObj.truncateValues = true;
	}

	// FIXME: need to sort rows once they're added if sort orders are specified
	if((parseObj["sortEnabled"])&&(parseObj["sortEnabled"]=="false")){
		widgetObj.sortEnabled = false;
	}


	var ctr = 0;
	/* 
		FIXME: this is an ugly hack, we really need to merge this 
		kind of parsing into __util__.parseNWComponents somehow
	*/
	if((parseObj["parseChildren"])&&(parseObj["parseChildren"]=="true")){
		with(__util__){
			domNode = stripWS(domNode);
			var fc = stripWS(domNode.firstChild);
			var bodyNode = stripWS(domNode.childNodes[1]);
			var headRow = stripWS(fc.firstChild);
		}
		// we require that there be a header in order to do anything else
		if(headRow.nodeName.toUpperCase()=="TR"){
			var tds = headRow.getElementsByTagName("td");
			if(tds.length == 0){
				tds = headRow.getElementsByTagName("th");
			}
			for(var x = 0; x < tds.length; x++){
				var ctd = tds[x];
				var cwidth = null;
				var cname = ctd.innerHTML;
				var tsc = "string";
				var tso = null;
				var tclass = null;
				if(__util__.hasAttr(ctd,"name")){
					cname = __util__.getAttr(ctd, "name");
				}
				if(__util__.hasAttr(ctd,"width")){
					var tcwidth = __util__.toInt(__util__.getAttr(ctd, "width"));
					cwidth = tcwidth;
					/*
					if(tcwidth!=0){
						cwidth = tcwidth;
					// FIXME: does this work?????
					}else if(!widgetObj.truncateValues){
						cwidth = __util__.getCSVal(ctd, "width");
					}
					*/
				}
				if(__util__.hasAttr(ctd,"sortComparator")){
					tsc = __util__.getAttr(ctd, "sortComparator");
				}
				if(__util__.hasAttr(ctd, "sortOrder")){
					tso = __util__.toInt(__util__.getAttr(ctd, "sortOrder"));
				}
				if(__util__.hasAttr(ctd, "class")){
					tclass = __util__.getAttr(ctd, "class");
				}
				widgetObj.addColumn(cname, cwidth, tsc, tso, tclass);
			}

			// now process widget data
			if(!__env__.is_IE){
				var drs = bodyNode.getElementsByTagName("tr");
				var trstr = "";
				for(var x = 0; x < drs.length; x++){
					var crow = drs[x];
					// crow.normalize();
					var crow = __util__.stripWS(drs[x]);
					var targs = [ctr];
					if(__util__.hasAttr(crow,"key")){
						targs[0] = [__util__.getAttr(crow,"key")];
					}
					var rdat = [];
					var cdat = [];
					for(var z = 0; z < crow.childNodes.length; z++){
						var celem = crow.childNodes[z];
						// var tnt = celem.nodeType;
						// if( (tnt != 8) && (tnt != 3) && (tnt != 12) ){
						rdat[rdat.length]=celem.innerHTML;
						if(__util__.hasAttr(celem, "class")){
							// FIXME: can we us || syntax for this assignment instead?
							cdat[cdat.length]=__util__.getAttr(celem, "class");
						}else{
							cdat[cdat.length]="";
						}
						// }
					}
					targs[1] = rdat;
					targs[2] = cdat;
					if(__env__.is_IE){
						trstr += widgetObj.addRow.apply(widgetObj, targs); 
					}else{
						widgetObj.addRow.apply(widgetObj, targs); 
					}
					ctr++;
				}
			}else{ // FIXME: HORRIBLE IE hack to speed creation in some instances
				widgetObj.domNode.style.visibility = "hidden";
				var spcr = widgetObj.displayTableNode.removeChild(widgetObj.displayTableNode.firstChild);
				var bdn = bodyNode.parentNode.removeChild(bodyNode);
				widgetObj.displayTableNode.parentNode.replaceChild(bdn, widgetObj.displayTableNode);
				widgetObj.displayTableNode = bdn;
				__theme__.setStyleByAttr(bdn, "main_font_size", "fontSize");
				__theme__.setStyleByAttr(bdn, "menu_bgcolor", "backgroundColor");
				var drs = bdn.getElementsByTagName("tr");
				for(var x = 0; x < drs.length; x++){
					var tkey = __util__.getAttr(drs[x], "key");
					drs[x].setAttribute("NW_table_key", tkey);
					// drs[x].onmousedown = widgetObj.lookupBaseStr+".setSelectedRow(this);";
					with(widgetObj){
						var tlen = rows.length;
						rows[tlen] = new Object();
						rows[tlen]["tr"]=drs[x];
						rows[tlen]["key"]=tkey;
					}
				}
				bdn.insertBefore(spcr, bdn.firstChild);
				widgetObj.domNode.style.visibility = "visible";
			}
		}
	}else{
		if(parseObj["nwComponents"]){
			var head = parseObj["nwComponents"]["header"][0];
			for(var x in head["nwComponents"]["columnHeader"]){
				var ch = head["nwComponents"]["columnHeader"][x];
				var cwidth = null;
				if(ch["width"]){
					cwidth = parseInt(ch["width"]);
				}
				var tsc = "string";
				if(ch["sortComparator"]){
					tsc = ch["sortComparator"];
				}
				var cso = null; 
				if(ch["sortOrder"]){
					cso = ch["sortOrder"];
				}
				var cclass = null; 
				if(ch["class"]){
					cclass = ch["class"];
				}
				widgetObj.addColumn(ch["columnName"], cwidth, tsc, cso, cclass);
				delete ch;
			}
			var body = parseObj["nwComponents"]["dataRecords"][0];
			// FIXME: what do we do if there aren't enough cells for each row? make some blank ones?
			for(var x in body["nwComponents"]["row"]){
				var cr = body["nwComponents"]["row"][x];
				// either use a synthetic key or the passed in one if it exists
				var targs = [ctr];
				if(cr["key"]){
					targs[0] = [cr["key"]];
				}
				targs[1] = cr["cell"];
				widgetObj.addRow.apply(widgetObj, targs); 
				ctr++;
			}
			delete head;
		}
	}
	delete parseObj;
	delete attrArr;
	// FIXME: are user sorting and sort-on-load tied?
	if((parseObj["sortOnLoad"])&&(parseObj["sortOnLoad"]=="true")){
		// FIXME: need to make this respect sort order prefs other than "0"
		var lse = widgetObj.sortEnabled;
		widgetObj.sortEnabled = true;
		for(var y = 0; y < widgetObj.columns.length; y++){
			var cn = widgetObj.columns[y].colHeaderNode;
			var tso = __util__.getAttr(cn, "sortOrder");
			if(!tso){
				tso = cn.getAttribute("sortOrder");
			}
			if(parseInt(tso)==0){
				widgetObj.sortColumn(cn);
			}
		}
		widgetObj.sortEnabled = lse;
	}

	with(widgetObj.domNode.style){
		if(parseObj["width"]){
			width = parseObj["width"]+"px";
		}
		if(parseObj["height"]){
			height = parseObj["height"]+"px";
		}
	}
	__sig__.connect(__env__, __env__.onResize, widgetObj, widgetObj.fixSizing);
	return widgetObj;
}

__scripts__.finalize(__config__.widgetPath+"gen_data_table_widget.js");
