/*
 * DHTML Kit Authored by Amberjack Software LLC ( www.amberjacksoftware.com)
 * Copyright  2000 Amberjack Software LLC 
 * Created on December 24, 2000
 *
 * DHtmlKit Component: MoverBox
 * Author:  Starling Consulting ( www.starlingconsulting.com )
 *          Casey Kochmer
 *          Eugene Ryser 
 * Version  1.00.12242000
 *
 * 	Modified By Alex Russell for netWindows compat and naming standards. 
 *	Added factory and object functions.
 *	Version NW_0.1
 *  
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version. 
 */

// $Id: mover_box_widget.js,v 1.25 2003/01/31 03:05:28 russcoon Exp $
// Filename:		mover_box_widget.js
// Purpose:			create and use mover-boxes
// Classes:			NW_WIDGET_mover_box
// Global Objects:	none
// Dependencies:	widget_core.js, theme_core.js, environ_core.js, component_registry.js
// Notes:			Since the original code for this is released under LGPL, this file is
//					also bound by the same license. The rest of netWindows, however, is 
//					covered by the MIT X-Consortium license.

with(__scripts__){
	provide(__config__.widgetPath+"mover_box_widget.js");
	require(__config__.corePath+"environ_core.js");
	require(__config__.corePath+"component_registry.js");
}

function NW_WIDGET_mover_box(domNode){
	this.parent = NW_widgetComponent;
	this.parent(domNode);
	delete this.parent;
	this.leftChooserBox = null;
	this.rightChooserBox = null;
	this.isTopLevelComp = false;
	this.isDraggable = false;
	this.addChoice = function(text, value){
		optionNode = document.createElement("option");
		optionNode.setAttribute("value", value);
		optionNode.innerHTML = text;
		this.leftChooserBox.appendChild(optionNode);
	}
	this.componentType = "mover_box";
	this.focus = function(){
		this.leftChooserBox.focus();
	}
}
NW_WIDGET_mover_box.prototype = new NW_widgetComponent; //deep inhertiance support
__widgets__.register("mover_box", "NW_WIDGET_FACTORY_inline_mover_box", "NW_WIDGET_FACTORY_mover_box");
 
function NW_WIDGET_FACTORY_inline_mover_box(domNode){
	domNode = __util__.stripWS(domNode);
	var mbObj = NW_WIDGET_FACTORY_mover_box();
	var attrArr = ['label', 'value', 'name', 'nwControlName'];
	var props = __util__.parseNWComponents(domNode, attrArr);
	if(props['nwControlName']||props['name']){
		var tname = props['nwControlName']||props['name'];
		with(mbObj.rightChooserBox){
			setAttribute("name", tname);
			name = tname;
			id = tname;
		}
	}
	for(var i in props["nwComponents"]["option"]){
		var to = props["nwComponents"]["option"][i];
		mbObj.addChoice(to['label'], to['value']);
	}
	return mbObj;
}

function NW_WIDGET_FACTORY_mover_box(){
	var gcfn = "(__components__.getComponentFromNode(this))";

	var iBN = document.createElement("table");
	iBN.setAttribute("cellpadding","0");	
	iBN.setAttribute("width","500");
	iBN.setAttribute("resize", "false");
	iBN.setAttribute("cellspacing","0");
	with(iBN.style){
		position = "relative";
		visibility = "visible";
		borderWidth = "1px 1px 1px 1px";
		padding = "2px 2px 2px 2px";
		borderStyle = "outset";
	}
	__theme__.setStyleByAttr(iBN, "main_bgcolor", "backgroundColor");
	__theme__.setStyleByAttr(iBN, "main_border_color", "borderColor");

	iTB = document.createElement("tbody");
	mTR = document.createElement("tr");
	mTR.style.width="100%";
	mTR.setAttribute("width","100%");
	mTR.setAttribute("height","20");

	lTD = document.createElement("td");
	lTD.setAttribute("width","40%");
	lTD.setAttribute("height","100%");
	lTD.setAttribute("align","right");

	var sFct = new NW_element_maker_obj("select");
	sFct.setEvent("onDblClick", "NW_WIDGET_mover_box_move_rows("+gcfn+".leftChooserBox, "+gcfn+".rightChooserBox);");
	lmBox = sFct.getElement();
	lmBox.setAttribute("multiple", "true");
	lmBox.setAttribute("size","10");
	lmBox.id="TT_ls_Choices";
	lmBox.style.width = "200px";
	
	lTD.appendChild(lmBox);
	
	mTD = document.createElement("td");
	mTD.setAttribute("width","20%");
	mTD.setAttribute("height","100%");
	mTD.setAttribute("align","center");

	var tbFct = new NW_element_maker_obj("input");
	tbFct.setEvent("onblur", "f_MoverFastEntry(this,"+gcfn+".leftChooserBox, "+gcfn+".rightChooserBox)");
	mIncludeTextBox = tbFct.getElement();
	mIncludeTextBox.setAttribute("type", "text");
	
	var incFct = new NW_element_maker_obj("input");
	incFct.setEvent("onClick", "NW_WIDGET_mover_box_move_rows("+gcfn+".leftChooserBox, "+gcfn+".rightChooserBox);");
	mIncludeButton = incFct.getElement();
	with(mIncludeButton){
		setAttribute("type", "button");
		setAttribute("value", "include");
		value = "include"; // konq3 fix
		style.width = "80px";
		style.fontSize = "11px";
	}

	var excFct = new NW_element_maker_obj("input");
	excFct.setEvent("onClick", "NW_WIDGET_mover_box_move_rows("+gcfn+".rightChooserBox, "+gcfn+".leftChooserBox );");
	mExcludeButton = excFct.getElement();
	with(mExcludeButton){
		setAttribute("type", "button");
		setAttribute("value", "exclude");
		value = "exclude"; // knoq3 fix
		style.width = "80px";
		style.fontSize = "11px";
	}


	var excbFct = new NW_element_maker_obj("input");
	excbFct.setEvent("onblur", "f_MoverFastEntry(this, "+gcfn+".rightChooserBox, "+gcfn+".leftChooserBox);");
	mExcludeTextBox = excbFct.getElement();
	mExcludeTextBox.setAttribute("type", "text");
	
	with(mTD){
		appendChild(mIncludeTextBox);
		appendChild(document.createElement("br"));
		appendChild(mIncludeButton);
		appendChild(document.createElement("br"));
		appendChild(mExcludeButton);
		appendChild(document.createElement("br"));
		appendChild(mExcludeTextBox);
	}
	
	rTD = document.createElement("td");
	rTD.setAttribute("width","40%");
	rTD.setAttribute("height","100%");

	var rmbFct = new NW_element_maker_obj("select");
	rmbFct.setEvent("onDblClick","NW_WIDGET_mover_box_move_rows( "+gcfn+".rightChooserBox, "+gcfn+".leftChooserBox );");
	rmBox = rmbFct.getElement();
	rmBox.setAttribute("multiple", "true");
	rmBox.style.width = "200px";
	rmBox.style.top = "0px";
	rmBox.setAttribute("size","10");
	rmBox.style.right = "0px";

	rTD.appendChild(rmBox);
	
	mTR.appendChild(lTD);
	mTR.appendChild(mTD);
	mTR.appendChild(rTD);
	iTB.appendChild(mTR);

	iBN.appendChild(iTB);

	var boxObj = new NW_WIDGET_mover_box(iBN); //cross-link the display and logic layers
	boxObj.leftChooserBox = lmBox;
	boxObj.rightChooserBox = rmBox;
	iBN.setAttribute("NW_compID", boxObj.globIdNum);
	iBN.id = boxObj.globIdNum;
	return boxObj;
}
 
 
/* This Function Moves rows from one list box to another */
function NW_WIDGET_mover_box_move_rows(active_box,dest_box)
{ for(var i=0; i<active_box.options.length; i++) 
	{if(active_box.options[i].selected && active_box.options[i].value != "" && active_box.options[i].value != "#")
		{  
			var luo_option = new Option();
			luo_option.value = active_box.options[i].value;
			luo_option.text  = active_box.options[i].text;
			dest_box.options[dest_box.options.length] = luo_option;
			active_box.options[i].value = "";
			active_box.options[i].text  = "";
   		}
	}
	Remove_Empty_Rows(active_box);
	Remove_Empty_Rows(dest_box);
	NW_WIDGET_mover_box_Sort_ListBox(dest_box);
	dest_box.selectedIndex = -1;
	active_box.selectedIndex = -1;
}

/* This Function Sorts a List box */
function NW_WIDGET_mover_box_Sort_ListBox(box)
{	var temp      = new Object();
	var li_count  = box.options.length;
	for(var x=0; x < li_count; x++)
	{
		for(var y=(x+1); y<li_count; y++)
		{   
			if(box.options[x].text > box.options[y].text)
			{
				temp.text = box.options[x].text;
				temp.value = box.options[x].value;
				box.options[x].text  = box.options[y].text;
				box.options[x].value = box.options[y].value;
				box.options[y].text  = temp.text;
				box.options[y].value = temp.value;
			}
   		}
	}
}

/* This Function removes empty lines from the list box */
function Remove_Empty_Rows(box)
{   var li_count = box.options.length ;
	var li_offset= 0;
    var li_total = 0;
	for(var li_next  = 0; (li_next + li_offset) < (li_count); li_next++) 
	{ while(box.options[li_next + li_offset].value == "")
		{   li_offset++;
			if ((li_next + li_offset) >= li_count) break;
	    }
	  if ((li_next + li_offset) >= li_count) break;
	  box.options[li_total].value = box.options[li_next + li_offset].value;
	  box.options[li_total].text  = box.options[li_next + li_offset].text;
	  li_total++	  
	}	
	box.options.length = (li_count - li_offset);  
}	

function f_MoverFastEntry(atxt_data, alb_FromBox, alb_ToBox)
{   
	var ls_data = atxt_data.value;
	if (ls_data.length > 0)
	{	var ls_match = /(\b\w+\b)+/;
		var lobj_results = ls_match.exec(ls_data);
		while ( lobj_results != null )
		{ls_data = ls_data.slice(lobj_results.lastIndex.valueOf());
 		 var lreg_pattern = new RegExp(("^" + lobj_results[0]) ,"i");
 			for(var i=0; i<alb_FromBox.options.length; i++) 
 			{ if(lreg_pattern.test(alb_FromBox.options[i].text))
 				{alb_FromBox.options[i].selected = true;}
  			}
		 lobj_results = ls_match.exec(ls_data); 
		}
	NW_WIDGET_mover_box_move_rows(alb_FromBox,alb_ToBox);
	atxt_data.value = "";
	}
	//if (auo_next_focus.toString() == "[object]") auo_next_focus.focus();
}

function NW_WIDGET_mover_box_f_SelectAll(Selected)
{   for(var i=0; i<Selected.options.length; i++) 
 	{ Selected.options[i].selected = true; }
}

function NW_WIDGET_mover_box_f_DeSelectAll(Selected)
{   for(var i=0; i<Selected.options.length; i++) 
 	{ Selected.options[i].selected = false;	}
}

__scripts__.finalize(__config__.widgetPath+"mover_box_widget.js");
