// $Id: progress_bar_widget.js,v 1.77 2002/10/16 23:01:32 russcoon Exp $
// Copyright (c) 2000-2002 Alex Russell
// Licensed under the Academic Free License version 1.1

// Filename:		progress_bar_widget.js
// Purpose:		defines the generic slider bar and a few subclasses
// Classes:		NW_WIDGET_gen_slider_bar
// Global Objects:	none
// Dependencies:	widget_core.js, utility_core.js,
//			theme_core.js, environ_core.js, component_registry.js

with(__scripts__){
	provide(__config__.widgetPath+"slider_bar_widget.js");
	require(__config__.corePath+"widget_core.js");
	require(__config__.corePath+"utility_core.js");
	require(__config__.corePath+"theme_core.js");
	require(__config__.corePath+"environ_core.js");
}

function NW_WIDGET_progress_bar(domNode, barNode, msgNode, barWidth, minVal, maxVal){
	//general NW component attributes
	this.parent = NW_widgetComponent;
	this.parent(domNode);
	delete this.parent;
	this.isTopLevelComp = false;
	this.isDraggable = false;
	this.componentType = "progress_bar";
	this.resize = null;

	//slider bar specific settings
	this.barNode = barNode; // domNode that holds the bar to be sized
	this.msgNode = msgNode; // text describing process, settable
	this.minVal = minVal;
	this.maxVal = maxVal;
	this.barWidth = barWidth; //size (in pixels) of the visible bar
	this.currentVal = null;
	this.setVal = function(currVal){ // change the current value
		var currPcnt = null;
		var currWidth = null;
		this.currentVal = currVal;
		currPcnt = ((this.currentVal - minVal)/maxVal);
		currWidth = currPcnt * barWidth;
		this.setWidth(currWidth);
		if(this.currentVal>=maxVal){
			this.destructor();
		}
	}

	this.setMsg = function(msgStr){
		this.msgNode.innerHTML = "";
		this.msgNode.appendChild(document.createTextNode(msgStr));
	}
	
	// not rocket science...
	this.setWidth = function(width){
		this.barNode.style.width = parseInt(width)+"px";
		if(!__env__.is_IE){
			//fix moz visibility bug by resizing container
			this.domNode.style.width = (parseInt(this.domNode.style.width)+1)+"px";
			this.domNode.style.width = (parseInt(this.domNode.style.width)-1)+"px";
		}
	}
	
	//destructor methods
	// **** should these be moved to the general component def? ****
	this.destructorCode = "this.remove();";
	this.setDestructor = function(code){
		this.destructorCode = code; }
	
	this.appendDestructor = function(code){
		this.destructorCode = code; }

	this.prependDestructor = function(code){
		this.destructorCode = code + this.destructorCode; }

	this.destructor = function(){
		eval(this.destructorCode); }
}
NW_WIDGET_progress_bar.prototype = new NW_widgetComponent; //deep inheritance support
__widgets__.register("progress_bar", "NW_WIDGET_INLINE_FACTORY_progress_bar", "NW_WIDGET_FACTORY_progress_bar");

// to do: add "message" field to allow users to set some text durring the progress bar's load
function NW_WIDGET_FACTORY_progress_bar(minVal, maxVal, pwidth, pheight){//optional 5th argument (string) is text of message to display
									//optional 6th argument (int) specifies height of bar
	
	var barWidth = parseInt(pwidth) - 10;
	var BDN = null;
	var bdmaker = new NW_element_maker_obj("div");
	BDN =  bdmaker.getElement();
	with(BDN.style){
		width = parseInt(pwidth) + "px";
		height = parseInt(pheight) + "px";
		position = "relative";
		visibility = "visible";
		borderWidth = "1px 1px 1px 1px";
		padding = "2px 2px 2px 2px";
		borderStyle = "outset";
	}
	__theme__.setStyleByAttr(BDN, "main_bgcolor", "backgroundColor");
	__theme__.setStyleByAttr(BDN, "main_border_color", "borderColor");
	
	var msgNode = null;
	var msgmaker = new NW_element_maker_obj("div");
	msgNode =  msgmaker.getElement();
	if((arguments.length>4)&&(arguments[4]!=null)){//handle message, if passed
		msgNode.appendChild(document.createTextNode(arguments[4]));
	}else{
		msgNode.appendChild(document.createTextNode("Loading..."));
	}
	var bHeight = null;
	if((arguments.length>5)&&(arguments[5]!=null)){//set bar height
		bHeight = parseInt(arguments[5]);
	}else{
		bHeight = 20;
	}
	BDN.appendChild(msgNode);
	
	var oBN = null;
	var obmaker = new NW_element_maker_obj("div");
	oBN =  obmaker.getElement();
	with(oBN.style){
		height = (bHeight + 2) + "px";
		width = (barWidth+2) + "px";
		borderWidth = "1px 1px 1px 1px";
		borderStyle = "inset";
	}
	__theme__.setStyleByAttr(oBN, "menu_bgcolor", "backgroundColor");
	__theme__.setStyleByAttr(oBN, "menu_border_color", "borderColor");
	BDN.appendChild(oBN);

	var iBN = null;
	var ibmaker = new NW_element_maker_obj("div");
	iBN =  ibmaker.getElement();
	with(iBN.style){
		height = bHeight + "px";
		width = "0px";
		borderWidth = "1px 1px 1px 1px";
		borderStyle = "outset";
	}
	__theme__.setStyleByAttr(iBN, "content_bgcolor", "backgroundColor");
	__theme__.setStyleByAttr(iBN, "content_border_color", "borderColor");
	oBN.appendChild(iBN);
	
	var barObj = new NW_WIDGET_progress_bar(BDN, iBN, msgNode, barWidth, minVal, maxVal);
	BDN.setAttribute("NW_compID", barObj.globIdNum);
	BDN.id = barObj.globIdNum;
	return barObj;
}


/* gen format for defining a progress bar inline:
	<div nwType="progress_bar">
		<div nwProperty="minVal">0</div>
		<div nwProperty="maxVal">100</div>
		<div nwProperty="width">300</div>
		<div nwProperty="height">38</div>
		<div nwProperty="msg">from inline constructor</div>
		<div nwProperty="barheight">15</div>
	</div>

the following also works:
	
	<div nwType="progress_bar" minVal="0" maxVal="100" width="100" height="100" msg="loading" barheight="15"></div>

*/
function NW_WIDGET_INLINE_FACTORY_progress_bar(domNode){
	var attrArr = ['minVal', 'maxVal', 'width', 'height', 'msg', 'barheight'];
	var props = __util__.parseNWProps(domNode, attrArr);
	var tMin = props["minVal"];
	var tMax = props["maxVal"];
	var tW = props["width"];
	var tH = props["height"];
	var tMsg = props["msg"];
	var tBH = props["barheight"];
	if((tBH!=null)&&(tBH!="")){
		return NW_WIDGET_FACTORY_progress_bar(tMin, tMax, tW, tH, tMsg, tBH);
	}else{
		return NW_WIDGET_FACTORY_progress_bar(tMin, tMax, tW, tH, tMsg);
	}
}

__scripts__.finalize(__config__.widgetPath+"slider_bar_widget.js");
