// $Id: split_pane_widget.js,v 1.18 2003/02/22 04:38:00 russcoon Exp $
// Copyright (c) 2000-2002 Alex Russell
// Licensed under the Academic Free License version 1.1

// Filename:		progress_bar_widget.js
// Purpose:		defines the split pane widget
// Classes:		NW_WIDGET_button
// Global Objects:	none
// Dependencies:	widget_core.js, utility_core.js,
//			theme_core.js, environ_core.js, component_registry.js

with(__scripts__){
	provide(__config__.widgetPath+"split_pane_widget.js");
	require(__config__.corePath+"widget_core.js");
	require(__config__.corePath+"utility_core.js");
	require(__config__.corePath+"theme_core.js");
	require(__config__.corePath+"environ_core.js");
	require(__config__.widgetPath+"content_area_widget.js");
}

function NW_WIDGET_split_pane(domNode){
	//set inheritance
	this.parent = NW_widgetComponent;
	this.parent(domNode);
	delete this.parent;
	//general NW component attributes
	this.componentType = "split_pane";
	this.isTopLevelComp = false;
	this.isDraggable = false;
	this.resize = null;

	this.dragBar = null;
	this.dragBarWidth = 3;
	this.isvert = true;
	this.canResize = false;
	this.pane1obj = null;
	this.pane2obj = null;
	this.lbph = 0; // only needed conditionaly
	this.ltph = 0; 

	// signals
	this.resizePaneX = function(width){}
	this.resizePaneY = function(height){}

	this.fixBottomPane = function(){
		// pull out the computed height of the entire widget
		if(!this.isvert){
			if(!__env__.is_IE){
				var ph = __util__.getCSVal(this.domNode, 'height');
				var bw=4; // combined width of pane borders
			}else{
				var tpn = this.domNode.parentNode;
				while(parseInt(tpn.offsetHeight)==0){
					tpn = tpn.parentNode;
				}
				var ph = parseInt(tpn.offsetHeight);
				var bw=4; // for wrong IE box model calcs
			}
			// the top pane will always have a "regular" style value set
			var tph = parseInt(this.pane1obj.domNode.style.height);
			if((ph!=this.lbph)||(tph!=this.ltph)){
				this.lbph=ph; this.ltph=tph;
				this.pane2obj.domNode.style.height = (((ph-parseInt(this.pane1obj.domNode.style.height))-parseInt(this.dragBar.style.height))-bw)+"px";
			}
		}
	}

	this.doOnPlacement = function(){
		this.fixBottomPane();
	}

	// handle internal resizing
	this.doPaneResize = function(evt){
		if(this.canResize){
			if(this.isvert){
				var pw = parseInt(this.pane1obj.domNode.style.width);
				var diffx = parseInt(__env__.lastX) - parseInt(__env__.currentX);
				if((pw>0)||(diffx<0)){
					__sig__.emit(this.resizePaneX, diffx);
				}
			}else{
				var ph = parseInt(this.pane1obj.domNode.style.height);
				var diffy = parseInt(__env__.lastY) - parseInt(__env__.currentY);
				if((ph>0)||(diffy<0)){
					__sig__.emit(this.resizePaneY, diffy);
				}
			}
		}
		if(!this.isvert){
			this.fixBottomPane();
		}
		return true;
	}
}
NW_WIDGET_split_pane.prototype = new NW_widgetComponent; //deep inhertiance support
__widgets__.register("split_pane", null, "NW_WIDGET_FACTORY_split_pane");

function NW_WIDGET_pane_class(node){
	this.domNode = node;
	this.contentObj = NW_WIDGET_FACTORY_content_area();
	if(__env__.is_IE){
		this.contentObj.domNode.style.position = "relative";
	}
	this.contentNode = this.contentObj.contentNode;
	with(this.contentObj.domNode.style){
		width="99%";
		height="99%";
	}
	this.domNode.appendChild(this.contentObj.domNode);
	//__theme__.setStyleByAttr(this.domNode, "content_bgcolor", "backgroundColor");
	//this.domNode.style.border = "1px solid #d4d4d4";
	this.resizeXby = function(diff){ 
		with(this.domNode.style){
			width = (parseInt(width)-diff)+"px"; 
		}
	}
	this.resizeYby = function(diff){ 
		with(this.domNode.style){
			height = (parseInt(height)-diff)+"px"; 
		}
	}
}

// factory function for split panes
function NW_WIDGET_FACTORY_split_pane(){

	// create DOM nodes
	var mmaker = new NW_element_maker_obj("div");
	var gfcstr = "__components__.getComponentFromNode(this)";
	mmaker.setEvent("onMouseMove", gfcstr+".doPaneResize(event);");
	mmaker.setEvent("onMouseUp", gfcstr+".canResize=false;");
	var main = mmaker.getElement();
	with(main.style){
		width = "100%";
		height = "100%";
		position = "absolute";
		//overflow = "hidden";
		borderWidth = "1px 1px 1px 1px";
		borderStyle = "solid"
	}
	__theme__.setStyleByAttr(main, "menu_border_color", "borderColor");

	var pane = new NW_WIDGET_split_pane(main);
	main.setAttribute("NW_compID", pane.globIdNum);
	main.id = pane.globIdNum;
	// we default to a vertical split bar, existance of optional first argument turns horizontal
	var flstr = "float: left; ";
	var pstr = "overflow: auto; height: 99%; ";
	if(arguments.length>0){ 
		if(arguments[0]==false){
			pane.isvert = false;
			flstr = "float: top;";
			pstr = "overflow: hidden; width: 99%; ";
		}
	}

	// 2nd argument specifies a value for the first significant dimension
	// if the pane is vertical, this will indicate that the height of the
	// top pane will be the value of this argument (in pixels), if the pane
	// is horizontal, then the left pane will be Xpx wide where X is arg 2
	var sigDim = "100";
	if(arguments.length>1){
		sigDim = arguments[1];
	}

	// optional 3rd argument specified whether or not the panes should be 
	// resizeable. The default is true, pass a string "false" to override.
	var isResizeable = true;
	if(arguments.length>2){
		if(arguments[2]==false){ isResizeable = false; }
	}
	
	if(arguments.length>3){
		pane.dragBarWidth = parseInt(arguments[3]);
	}
	
	var p1 = document.createElement("div");
	__util__.setNodeStyle(flstr+pstr, p1);
	p1obj = new NW_WIDGET_pane_class(p1);

	if(isResizeable){
		if(pane.isvert){
			__sig__.connect(pane, pane.resizePaneX, p1obj, p1obj.resizeXby);
		}else{
			__sig__.connect(pane, pane.resizePaneY, p1obj, p1obj.resizeYby);
		}
	}

	var barmaker = new NW_element_maker_obj("div");
	barmaker.setEvent("onMouseDown", gfcstr+".canResize=true;");
	var bar = barmaker.getElement();
	pane.dragBar = bar;
	__util__.setNodeStyle(pstr+flstr+"border: 1px/solid/#d4d4d4; backgroundColor: #f6f6f6;", bar);
	__theme__.setStyleByAttr(bar, "main_border_color", "borderColor");
	__theme__.setStyleByAttr(bar, "main_bgcolor", "backgroundColor");

	var p2 = document.createElement("div");
	__util__.setNodeStyle(pstr, p2);
	//p2.style.position = "absolute";
	p2obj = new NW_WIDGET_pane_class(p2);

	// set up default formatting, 50% to each pane
	var pheight = 0;
	var pwidth = 0;
	if(pane.isvert){
		bar.style.width = (pane.dragBarWidth-2)+"px";
		p1obj.domNode.style.width = sigDim+"px";
	}else{
		p1obj.domNode.style.height= sigDim+"px";
		bar.style.height = (pane.dragBarWidth-2)+"px";
	}

	main.appendChild(p1);
	main.appendChild(bar);
	main.appendChild(p2);
	pane.pane1obj = p1obj;
	pane.pane2obj = p2obj;

	return pane;
}

// FIXME: need to add an inline constructor for split panes!

__scripts__.finalize(__config__.widgetPath+"split_pane_widget.js");
