// $Id: text_expander_widget.js,v 1.13 2003/02/22 05:22:19 russcoon Exp $
// Copyright (c) 2000-2002 Alex Russell
// Licensed under the Academic Free License version 1.1

// Filename:		text_expander_widget.js
// Purpose:		auto-completes some text
// Classes:		NW_WIDGET_text_expander
// Global Objects:	none
// Dependencies:	widget_core.js, utility_core.js,
//			theme_core.js, environ_core.js, component_registry.js

with(__scripts__){
	provide(__config__.widgetPath+"text_expander_widget.js");
	require(__config__.corePath+"widget_core.js");
	require(__config__.corePath+"utility_core.js");
	require(__config__.corePath+"theme_core.js");
	require(__config__.corePath+"environ_core.js");
}

function NW_WIDGET_text_expander(domNode){
	//set inheritance
	this.parent = NW_inputWidget;
	this.parent(domNode);
	delete this.parent;
	this.componentType = "text_expander";
	this.isTopLevelComp = false;
	this.isDraggable = false;
	this.resize = null;
	this.abbrevEnabled = false;
	this.abbrevPassive = false;
	this.primeDoExpand = false;
	this.primeDoTab = false;
	this.tabForward = false;
	this.watchedElement = null;
	this.tabTo = null;
	this.lastTabIndex = 0;
	this.ltval = "";
	this.ctval = "";

	this.focus = function(){
		if(this.watchedElement){
			this.watchedElement.focus();
		}
	}

	this.matchIgnoreCase = true;

	// FIXME: need to add mouse hinting for collapse

	// this is a 2-deep trie structure for quick parring down of options when doing prefix matching on words
	this.dataArr = new Array();

	this.addOption = function(abbrev, value){
		this.dataArr[abbrev] = value;
	}

	this.populateOptions = function(optsList){
		for(var x in optsList){
			if(optsList[x].constructor != [].constructor){
				this.addOption(optsList[x], optsList[x]);
			}else{
				this.addOption(optsList[x][0], optsList[x][1]);
			}
		}
	}

	this.setCaretLocation = function(location){
		var loc = __util__.toInt(location);
		var twe = this.watchedElement;
		// borrowed from http://www.faqts.com/knowledge_base/view.phtml/aid/13562/fid/130
		if(twe.setSelectionRange){
			twe.focus();
			twe.setSelectionRange(loc, loc);
		}else if(twe.createTextRange){
			var range = twe.createTextRange();
			range.collapse(true);
			range.moveEnd('character', loc);
			range.moveStart('character', loc);
			range.select();
		}else{//otherwise try the event-creation hack (our own invention)
			// do we need these?
			twe.value = twe.value;
			twe.blur(); twe.focus();
			// figure out how far back to go
			var dist = __util__.toInt(twe.value.length)-loc;
			var tchar = String.fromCharCode(37);
			var tcc = tchar.charCodeAt(0);
			for(var x = 0; x < dist; x++){
				var te = document.createEvent("KeyEvents");
				te.initKeyEvent("keypress", true, true, null, false, false, false, false, tcc, tcc);
				this.watchedElement.dispatchEvent(te);
			}
		}
	}

	this.doExpand = function(){
		for(var x = 0; x < this.ctval.length; x++){
			if(this.ctval.charAt(x)!=this.ltval.charAt(x)){
				// treat x as the index of our spacer
				var lstr = this.ctval.slice(0, x);
				if((lstr.length + 1) != this.ctval.length){
					// var fhalf = lstr.slice(0, lstr.lastIndexOf(" "));
					var fhalf = lstr;

					while(fhalf.lastIndexOf(" ")==(fhalf.length-1)){
						fhalf = fhalf.slice(0, fhalf.length-1);
					}
					var lhalf = this.ctval.slice(x);
					if(lstr.lastIndexOf(" ")!=-1){
						var tindex = fhalf.lastIndexOf(" ");
						var ttoken = fhalf.slice(tindex).replace(/\s*/g,"");
						if(this.dataArr[ttoken]){
							this.watchedElement.value = fhalf.slice(0, tindex) + " " +  this.dataArr[ttoken] + lhalf;
							this.setCaretLocation(this.watchedElement.value.length-lhalf.length);
						}
					}else{
						// otherwise we're at the beginning of the string
						var ttoken = lstr.replace(/\s*/g,"");
						if(this.dataArr[ttoken]){
							this.watchedElement.value = this.dataArr[ttoken] + " " + lhalf;
							this.setCaretLocation(this.dataArr[ttoken].length-1);
						}
					}
				}else if(lstr.indexOf(" ")==-1){
					var fstr = "";
					var kstr = lstr;
					if(lstr.lastIndexOf("\n")!=-1){
						var tla = lstr.lastIndexOf("\n");
						fstr = lstr.slice(0, tla+1);
						kstr = kstr.slice(tla);
					}
					kstr = kstr.replace(/\s*/g,"");
					if(this.dataArr[kstr]){
						this.watchedElement.value = fstr + this.dataArr[kstr];
					}
				}else{
					// handle all other end-of-text replacements
					var warr  = lstr.split(" ");
					var spaceWon = true;
					var endpt = lstr.lastIndexOf(" ");
					if(lstr.lastIndexOf("\n")>endpt){
						spaceWon = false;
						endpt = lstr.lastIndexOf("\n");
					}
					if(warr.length>0){
						var cel  = warr[warr.length-1];
						cel = cel.split("\n");
						var lestr = "";
						for(x in cel){
							if(cel[x]==""){
								lestr += "\n";
							}
						}
						if((cel.length == 0)||(spaceWon)){
							lestr = " ";
						}
						cel = cel[cel.length-1];
						if(this.dataArr[cel]){
							this.watchedElement.value = lstr.slice(0, endpt) + lestr + this.dataArr[cel];
						}
					}
				}
				break;
			}
		}
	}

	__sig__.addBareSignal(this, this.doExpand);

	this.checkTab= function(evt){
		if((evt.keyCode)&&(evt.keyCode==9)&&(__env__.is_IE)){
			if(this.tabTo!=null){
				__util__.cancelEvent(evt);
				evt.returnValue = false;
				// start movement through the text. We don't have a way to tell where the caret is right now, so start from the back and move forward.
				var cval = this.watchedElement.value;
				if(this.lastTabIndex>=cval.length){
					this.lastTabIndex = 0;
				}
				var cval = this.watchedElement.value;
				var ci = cval.indexOf(this.tabTo, this.lastTabIndex+1);
				this.lastTabIndex = ci;
				if(this.lastTabIndex == -1){
					if(this.tabForward){
						evt.returnValue = true;
						return true;
					}
					var ci = cval.indexOf(this.tabTo, 0);
					this.lastTabIndex = ci;
				}
				this.setCaretLocation(this.lastTabIndex);
			}
		}
	}

	__sig__.addBareSignal(this, this.checkTab);

	this.valueChange = function(value){
		if(this.ctval!=value){
			this.ltval = this.ctval;
			this.ctval = value;
			var charCode = null;
			if(arguments.length>1){
				var evt = arguments[1];
				if(__env__.is_IE){
					charCode = evt.keyCode;
				}else{
					charCode = evt.charCode;
				}
				if(__util__.toInt(charCode) == 32){
					this.primeDoExpand = true;
				}else{
					this.primeDoExpand = false;
				}
			}
		}
	}
	__sig__.addBareSignal(this, this.valueChange);

	this.checkExpand = function(value, evt){
		if((this.primeDoExpand)&&(this.abbrevEnabled)){
			this.primeDoExpand = false;
			this.valueChange(value);
			this.doExpand();
		}
	}
	__sig__.addBareSignal(this, this.checkExpand);


	this.doOnPlacement = function(){
		if((this.abbrevEnabled)&&(this.abbrevPassive)){
			// FIXME: substitute on all strings in the document
		}
	}
}

NW_WIDGET_text_expander.prototype = new NW_inputWidget; //deep inhertiance support
__widgets__.register("text_expander", "NW_WIDGET_INLINE_FACTORY_text_expander", "NW_WIDGET_FACTORY_text_expander");

// factory function for split panes
function NW_WIDGET_FACTORY_text_expander(){

	var gfcstr = "__components__.getComponentFromNode(this)";

	// create DOM node
	var imkr  = new NW_element_maker_obj("textarea");
	// enable our keystroke handlers
	if(!__env__.is_konq){
		imkr.setEvent("onKeyDown", "__sig__.emit("+gfcstr+".checkTab, event);");
		imkr.setEvent("onKeyPress", "__sig__.emit("+gfcstr+".valueChange, this.value, event);");
		imkr.setEvent("onKeyUp", "__sig__.emit("+gfcstr+".checkExpand, this.value, event);");
	}
	var input = imkr.getElement();
	if(__env__.is_konq){
		input.onkeyup = function(event){
			var src = __util__.evtTarget(event);
			var te = __components__.getComponentFromNode(src);
			te.ltval = te.ctval;
			te.ctval = src.value;
			// figure out of the last pressed key was a space.
			for(var x = 0; x < te.ctval.length; x++){
				if(te.ctval.charAt(x)!=te.ltval.charAt(x)){
					if(te.ctval.charAt(x)==" "){
						te.doExpand(event);
					}
					break;
				}
			}
		}
	}
	with(input.style){
		if(__config__.applyThemedStyles){
			// width = "100px";
			borderWidth = "1px";
			borderStyle = "solid";
			letterSpacing = "1px";
			paddingLeft = "5px";
		}
	}
	with(__theme__){
		setStyleByAttr(input, "head_bar_bgcolor", "borderColor");
		setStyleByAttr(input, "main_bgcolor", "backgroundColor");
		setStyleByAttr(input, "menu_font_size", "fontSize");
		setStyleByAttr(input, "menu_font_color", "color");
	}

	if(__env__.is_IE){
		input.setAttribute("wrap","off");
	}
	input.setAttribute("class","TextExpander-entry-input");
		
	var te = new NW_WIDGET_text_expander(input);
	te.watchedElement = input;
	input.setAttribute("NW_compID", te.globIdNum);
	input.id = te.globIdNum;
	return te;
}

function NW_WIDGET_INLINE_FACTORY_text_expander(domNode){
	// FIXME: need to implement abbrevPassive, abbrevFocus, and tabTo
	var attrArr = ['url', 'nwControlName', 'matchIgnoreCase', 'name', 'cols', 'rows', 'style', 'shortcut', 'abbr', 'abbrev', 'label', 'value', 'abbrevMap', 'abbrevEnabled', 'tabTo', 'abbrevFocus', 'abbrevPassive', 'url', 'tabForward'];
	var multiValued = ['url'];
	var props = __util__.parseNWComponents(domNode, attrArr, multiValued);
	var te = NW_WIDGET_FACTORY_text_expander();

	if((props['tabForward'])&&(props['tabForward']=="true")){
		te.tabForward = true;
	}

	// default is false, so only check for true
	if((props['matchIgnoreCase'])&&(props['matchIgnoreCase']=="false")){
		te.matchIgnoreCase = false;
	}

	if((props['abbrevEnabled'])&&(props['abbrevEnabled']=="true")){
		te.abbrevEnabled = true;
	}

	if((__util__.hasAttr(domNode,'nwControlName'))||(props['name'])){
		var tname = __util__.getAttr(domNode,'nwControlName')||props['name'];
		with(te.watchedElement){
			setAttribute("name", tname);
			name = tname;
			id = tname;
		}
	}
	/* 	need to figure out a better way to set this that won't walk all 
		over properties we care about that the constructor set
	if(props["style"]){
		te.watchedElement.setAttribute("style",props["style"]);
	}
	*/
	if(props["cols"]){
		te.watchedElement.setAttribute("cols",props["cols"]);
	}
	if(props["rows"]){
		te.watchedElement.setAttribute("rows",props["rows"]);
	}

	if(props["tabTo"]){
		te.tabTo = props["tabTo"];
	}

	if(!__env__.is_konq){
		if(props["url"]){
			var urlarr = props["url"];
			for(var x in urlarr){
				try{
					te.populateOptions(eval(NW_getFileTxt(urlarr[x])));
				}catch(e){ stdout(e); }
			}
		}
	}

	if(props["abbrevMap"]){
		// we need to do our own parsing to preserve whitespace
		var map = __util__.parsePropStr(__util__.getAttr(domNode,"abbrevMap"), false);
		for(var key in map){
			te.addOption(key,map[key]);
		}
	}

	te.watchedElement.value = domNode.innerHTML;
        te.watchedElement.defaultValue = domNode.defaultValue;

	return te;
}

__scripts__.finalize(__config__.widgetPath+"text_expander_widget.js");
