// $Id: wizard_widget.js,v 1.19 2002/11/09 06:51:41 russcoon Exp $
// Copyright (c) 2000-2002 Alex Russell
// Licensed under the Academic Free License version 1.1

// Filename:		wizard_widget.js
// Purpose:		Provide the functionality to generate a multiple step form, 
//			with possible variances along the way depending on user input.
// Classes:		NW_WIDGET_wizard
// Global Objects:	none
// Dependencies:	widget_core.js, theme_core.js, environ_core.js, component_registry.js
// Author:  		Bill Ramsour

with(__scripts__){
	provide (__config__.widgetPath+"wizard_widget.js");
	require(__config__.corePath+"theme_core.js");
	require(__config__.corePath+"environ_core.js");
	require(__config__.widgetPath+"alert_box_widget.js");
}

// wizard class def
function NW_WIDGET_wizard(domNode) {
	this.parent = NW_widgetComponent;
	this.parent(domNode);
	delete this.parent;

	this.domNode = domNode;//top level container
	this.contentNode = null;//content container
	this.parentAlert = null; // only used if modal
	this.wizardTbl = null; // reference to the internal container, regarless of mode
	this.backButton = new NW_WIDGET_FACTORY_button("< Back", 70, 20);
	this.nextButton = new NW_WIDGET_FACTORY_button("Next >", 70, 20);
	this.isTopLevelComp = false;
	this.isDraggable = false;
	this.componentType = "wizard";
	this.resize = null;

	this.numPages = null;
	this.PageData = null;
	this.FormData = document.createElement("form");
	this.FormNameArr = new Array();
	this.currentPage = 0;
	this.domForm = null;
	this.maxPage = -1;	//farthest page reached, needed if "Back" is hit.

	this.getVisForm = function(){
		var nl = null;
		var sn = this.domNode;
		if(arguments.length==1){
			sn = arguments[0];
			nl = sn.childNodes.length;
		}else{
			nl = this.domNode.childNodes.length;
		}
		for(var x=0; x<nl;x++){
			var cn = sn.childNodes[x];
			if(cn.nodeName.toLowerCase()=="form"){
				return cn;
			}else{
				if(cn.hasChildNodes()){
					ret = this.getVisForm(cn);
					if(ret!=null){return ret;}
				}else{
					if(x==(nl-1)){ return null;}
				}
			}
		}
		return null; // no form in this.domForm
	}

	this.parseData = function(dataBlock){ // put inline block parser functions here.
		this.PageData = dataBlock;
		this.numPages = dataBlock['nwComponents']['page'].length;
		this.InitFormData();
		this.LoadCurrentPage();
	}

	this.loadURL = function(url){
		NW_wiz_queue.addRequest(this, url);
	}

	this.InitFormData = function() {
		this.FormData.setAttribute("name", "form_data");
		var counter = 0;
		for(var i = 0; i < this.PageData['nwComponents']['page'].length; i++) {
			var cpage = this.PageData['nwComponents']['page'][i];
			if (cpage['input']) {
				if (cpage['input']) {
					var inputLen = cpage['input'].length;
					for(var j = 0; j < inputLen; j++) {
						inputFormNode = document.createElement("input");
						var inputName = (new String(cpage['input'][j])).split(" ");
						inputFormNode.setAttribute('name', inputName[0]);
						this.FormNameArr[inputName[0]] = counter;
						this.FormData.appendChild(inputFormNode);
						counter++;
					}
				}
				/*
				if (currForm.Radio) {
					//I really hate radios
				}
				*/
			}
		}
	}

	this.LoadCurrentPage = function(){
		if(this.currentPage == 0){
			this.backButton.setInactive();
		}else{
			this.backButton.setActive();
		}

		var tcpage = this.PageData['nwComponents']['page'][this.currentPage];

		if(this.currentPage < this.numPages) {
			this.contentNode.innerHTML = tcpage['body'];
			this.domForm = this.getVisForm(this.contentNode);

			if (this.currentPage > this.maxPage) {
				this.maxPage = this.currentPage;
			} else if(tcpage['input']) {
				//Load up form stuff because this page has already been seen and it has forms
				if (tcpage['input']) {
					for(var i = 0; i < tcpage['input'].length; i++) {
						var inputName = tcpage['input'][i].split(" ");
						var savedForm = this.FormData.childNodes[this.FormNameArr[inputName[0]]];
						var changeForm = eval("this.domForm."+savedForm.name);
						if(changeForm.type == "checkbox" || changeForm.type == "Checkbox") {
							changeForm.checked = savedForm.checked;
						}else{
							changeForm.value = savedForm.value;
						}
					}
				}
			}
		} else {
			this.contentNode.innerHTML = "You input: <br>";
			this.contentNode.appendChild(this.FormData);
			this.nextButton.setText("Finish");
			this.nextButton.onClick = "__glob_comp_reg__.getComponent('"+this.globIdNum+"').Finish();";
		}
		
	}
	this.Finish = function() {
		stdout("you're finished!!!");
	}
	
	this.Advance = function() {
		//Take care of saving the form stuff into this.FormData
		if (this.PageData['nwComponents']['page'][this.currentPage].form) {
			var currForm = this.PageData.page[this.currentPage].form[0];
			if (currForm.input) {
				var inputLen = currForm.input.length;
				for(var i = 0; i < inputLen; i++) {
					var inputNode = eval("this.domForm."+currForm.input[i].value);
					var savedNode = this.FormData.childNodes[this.FormNameArr[inputNode.name]];

					if(!__env__.is_IE){savedNode.setAttribute('type', inputNode.type);}
					savedNode.setAttribute('value', inputNode.value);
					if(inputNode.type == "checkbox" || inputNode.type == "Checkbox") {	
						inputNode.checked = true;
						// this.FormData.childNodes[this.FormNameArr[inputNode.name]].setAttribute("checked", inputNode.checked);
						savedNode.checked = true;
						savedNode.checked = inputNode.checked;
						// this doesnt' come close to working
						// stdout(this.FormData.childNodes[this.FormNameArr[inputNode.name]].checked);
					}
				}
			} else if(currForm.Radio) {
			}
		}
		this.currentPage++;
		this.LoadCurrentPage();
	}
	
	
	this.goToNextPage = function(){
		var tcp = this.PageData['nwComponents']['page'][this.currentPage];
		if (tcp['nextFunction']) {
			var MoveOn = eval(tcp['nextFunction']);
			if(MoveOn) { this.Advance(); }
		}else{
			this.Advance();
		}
	}


	this.goToPrevPage = function(){
		if(this.currentPage == this.numPages) {
			this.nextButton.setText("Next >");
			this.nextButton.setOnClick("__glob_comp_reg__.getComponent('"+this.globIdNum+"').goToNextPage();");
		}
		this.currentPage--;
		this.LoadCurrentPage();
	}

}
NW_WIDGET_wizard.prototype = new NW_widgetComponent; //deep inhertiance support
__NW_widgets__.register("wizard", "NW_WIDGET_INLINE_FACTORY_wizard", "NW_WIDGET_FACTORY_wizard");

// order queue and data receptor for wizard widget data requests.
function NW_WIDGET_wizard_data_queue(){
	this.dataRcptName = 'wizard_widget_data';
	this.attributes = ['mode', 'header', 'body', 'input', 'nextFunction'];
	this.multiValued = ['input'];
	this.queueArr = new Array();
	this.currentReq = 0;
	this.lastReq = 0;
	this.addRequest = function(wizardObj, url){
		this.queueArr[this.lastReq]= wizardObj;
		// optional 3rd argument specifies target block, but none is expected
		var tgt = __data__.tgtNode;
		if(arguments.length==3){
			tgt = arguments[2];
		}
		__env__.addToPageQueue(url, tgt, false);
		if(this.queueArr.length>1){
			this.lastReq++;
		}
	}

	this.rcv = function(dataBlock){
		(this.queueArr[this.currentReq]).parseData(dataBlock);
		delete this.queueArr[this.currentReq];
		this.currentReq++;
	}
}
var NW_wiz_queue = new NW_WIDGET_wizard_data_queue();
__data__.addDataRcpt(NW_wiz_queue);


////////////////////////////// WIDGET FACTORY //////////////////////////////////
function NW_WIDGET_FACTORY_wizard(wtype){
	// we support 2 "modes" with this widget, defined by the type argument
	// they are "inline" for a wizard that is meant to be contained in a relatively
	// positioned container, and "modal" for a wizard that is contained in an alert box
	var pwidth = 200;
	var pheight = 325;
	var headerText = "Wizard";
	var gfcstr = "__components__.getComponentFromNode(this)";

	if(arguments.length>1){ pwidth = parseInt(arguments[1]);} // alert box width
	if(arguments.length>2){ pheight = parseInt(arguments[2]);} // alert box height
	if(arguments.length>3){ headerText= arguments[3];} // alert box header text

	// top level wizard node
	var itn = document.createElement("table" + __env__.ctxt);
	itn.setAttribute("height","100%");
	itn.setAttribute("width","100%");
	var itb = document.createElement("tbody" + __env__.ctxt);
	itn.appendChild(itb);
	// content row
	var ctr = document.createElement("tr" + __env__.ctxt);
	ctr.setAttribute("height","*");
	ctr.setAttribute("valign","top");
	itb.appendChild(ctr);
	// content column 
	var ctd = document.createElement("td" + __env__.ctxt);
	ctd.setAttribute("colspan","2");
	ctr.appendChild(ctd);

	// footer (button) row
	var ftr = document.createElement("tr" + __env__.ctxt);
	with(ftr){
		setAttribute("height","25");
		style.height = "25px";
		style.width = "100%";
		setAttribute("valign","top");
	}
	itb.appendChild(ftr);
	// left button column
	var lbtd = document.createElement("td" + __env__.ctxt);
	ftr.appendChild(lbtd);
	lbtd.setAttribute("align","left");
	// right button column
	var rbtd = document.createElement("td" + __env__.ctxt);
	ftr.appendChild(rbtd);
	rbtd.setAttribute("align","right");

	var wizObj = new NW_WIDGET_wizard(itn);
	itn.setAttribute("NW_compID", wizObj.globIdNum);
	itn.id = wizObj.globIdNum;
	with(wizObj){
		wizardTbl = itn; // doensn't change due to mode
		contentNode = ctd;
		backButton.setOnClick("__components__.getComponent('"+globIdNum+"').goToPrevPage();");
		nextButton.setOnClick("__components__.getComponent('"+globIdNum+"').goToNextPage();");
		lbtd.appendChild(backButton.domNode);
		rbtd.appendChild(nextButton.domNode);
		backButton.setInactive();
	}

	if(wtype=="modal"){
		var wizAlert = NW_WIDGET_FACTORY_alert_box(headerText, "", pheight, pwidth);	
		wizObj.parentAlert = wizAlert;
		wizObj.domNode = wizAlert.domNode;
		wizAlert.contentNode.innerHTML = "";
		wizAlert.contentNode.appendChild(itn);
	}
	
	return wizObj;
}

function NW_WIDGET_INLINE_FACTORY_wizard(domNode) {
	// var wizData = NW_genericParser(domNode, null, 'nType');
	var attrArr = ['mode', 'header', 'body', 'input', 'nextFunction'];
	var wizData = __util__.parseNWComponents(domNode, attrArr, ['input']);
	var wizard = NW_WIDGET_FACTORY_wizard(wizData['mode']);
	wizard.PageData = wizData;
	wizard.numPages = wizData['nwComponents']['page'].length;
	wizard.InitFormData();
	wizard.LoadCurrentPage();
	return wizard;
}

/////////////////////////// HELPER/VANITY FUNCTIONS ////////////////////////////
function NW_WIDGET_wizard_create_load_append(url){
	var wtype = "modal";
	var tempWiz = null;
	if(arguments.length == 2){
		//optional destination argument references location of wizard widget
		var dest = arguments[1];
		wtype = "inline";
	}
	tempWiz = NW_WIDGET_FACTORY_wizard(wtype);
	// use __data__ receptor to load the data
	NW_wiz_queue.addRequest(tempWiz, url);
	if(wtype=="inline"){
		dest.appendChild(tempWiz.domNode);
	}
}


__scripts__.finalize(__config__.widgetPath+"wizard_widget.js");
